import { Component, OnInit, Input, EventEmitter, Output } from '@angular/core';
import { UUID } from 'angular2-uuid';
import { UserAccountService } from '../../services/user-account/user-account.service';
import { FileUploader } from 'ng2-file-upload';

@Component({
  selector: 'flow-uploader',
  templateUrl: './flow-uploader.component.html',
  styleUrls: ['./flow-uploader.component.css']
})
export class FlowUploaderComponent implements OnInit {
  
  public id: any = UUID.UUID();

  private _uploadUrl: string;
  public loading: boolean;

  @Input()
  public uploader: any = new FileUploader({});

  public reader: FileReader = new FileReader();

  @Input()
  public pasteEnabled: boolean;

  private _autoUpload: boolean = true;
  
  
  public get autoUpload() {
    return this._autoUpload;
  }
  @Input()
  public set autoUpload(value) {
    this._autoUpload = value;
  }

  @Input()
  public labelclass: string;

  @Input()
  public showLoading: boolean;

  @Input()
  public set uploadUrl(value)
  {
    this._uploadUrl = value;
    this.uploader.setOptions({ url: value });
    
  }

  public get uploadUrl() {
    return this._uploadUrl;
  }

  @Output() 
  public fileAdded = new EventEmitter(true);

  @Output()
  public onUploadComplete = new EventEmitter(true);

  @Input()
  public uploadCompleted: boolean = false;

  @Input()
  public accept: string = '*.*'; 

  constructor(public userService: UserAccountService) { }


  ngOnInit() {
    if (this.pasteEnabled) {
      window.addEventListener('paste', this.handlePaste);
    }

    this.uploader.setOptions({
      authToken: 'Bearer ' + this.userService.token,
      'Content-Type': 'multipart/form-data'
    });
    
    this.uploader.onAfterAddingFile =  (item) => { 
        item.withCredentials = false; 
        if (this._autoUpload)
        {
          this.loading = true;
          this.uploader.uploadAll();

        }
        else
        {
          this.reader.onload=(event) => { this.fileAdded.emit(this.reader.result);};
          this.reader.readAsDataURL(item._file);
          
        }
      }
      this.uploader.onSuccessItem = (item, response, status, header) => { this.uploader.clearQueue(); this.onUploadComplete.emit({item: item, response: response, status: status, header: header}); this.loading = false; }
      this.uploader.onErrorItem = (item, response, status, header) => { this.uploader.clearQueue(); this.onUploadComplete.emit({item: item, response: response, status: status, header: header}); } 
  }

  handlePaste(f: any) {
    if (f.clipboardData.files.length > 0){
      this.uploader.addToQueue([f.clipboardData.files[0]]);
    }
  }
  ngOnDestroy() {
    window.removeEventListener('paste', this.handlePaste);
  }
}
