import { Component, OnInit, Input, forwardRef, HostListener } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';

const noop = () => {
};

export const CUSTOM_INPUT_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => FlowconceptIcheckComponent),
  multi: true
};

@Component({
  selector: 'icheck',
  templateUrl: './flowconcept-icheck.component.html',
  styleUrls: ['./flowconcept-icheck.component.css'],
  providers: [CUSTOM_INPUT_CONTROL_VALUE_ACCESSOR]
})
export class FlowconceptIcheckComponent implements OnInit, ControlValueAccessor {

  private innerValue: boolean = false;

  private onTouchedCallback: () => void = noop;
  private onChangeCallback: (_: any) => void = noop;

  get value(): any {
    return this.innerValue;
  };

  @Input()
  set value(v: any) {
    if (v !== this.innerValue) {
      this.innerValue = v;
      this.onChangeCallback(v);
    }
  }

  //Set touched on blur
  onBlur() {
    this.onTouchedCallback();
  }

  //From ControlValueAccessor interface
  writeValue(value: any) {
    if (value !== this.innerValue) {
      this.innerValue = value;
    }
  }

  //From ControlValueAccessor interface
  registerOnChange(fn: any) {
    this.onChangeCallback = fn;
  }

  //From ControlValueAccessor interface
  registerOnTouched(fn: any) {
    this.onTouchedCallback = fn;
  }


  @HostListener('click', ['$event'])
  onClick(e) {
    if (this.disabled)
      return;
    if (this.type == 'radio' && this.value)
      return;
    
    this.value = !this.value;
    
  }

  @Input()
  public disabled: boolean = false;
  @Input()
  public type: string = 'checkbox';

  @Input() 
  public hideLabel: boolean = false;

  @Input()
  public labelClass: string;
  @Input()
  public checkboxClass: string;

  constructor() { }

  ngOnInit() {
  }

}
