import { Component, OnInit, ChangeDetectorRef, NgZone, ApplicationRef } from '@angular/core';
import { GlobalService, TitleService, TranslationSharedService, UserAccountService } from '../../services/';
import { ActivatedRoute, Router } from '@angular/router';
import { DomSanitizer, SafeHtml, Title } from '@angular/platform-browser';
import { animations } from '../../animations/shared.animations';


@Component({
  selector: 'app-html-page',
  templateUrl: './html-page.component.html',
  styleUrls: ['./html-page.component.css'],
  animations: animations
})
export class HtmlPageComponent implements OnInit {

  public editorOpen: boolean = false;
  public saveSuccess: boolean = false;
  public editContent: boolean = false;
  public translation: any = {};
  public content: any;

  constructor (public translationService: TranslationSharedService, public route: ActivatedRoute, public changeDetectorRef: ChangeDetectorRef,   public sanitizer: DomSanitizer,
    public userService: UserAccountService,
    public router: Router,
    public zone: NgZone,
    public appRef: ApplicationRef,
    public titleService: TitleService,
    public globalService: GlobalService
  ) { 

  }

  ngOnInit() {
    this.route.params.subscribe(res => {
      this.populatePage(res['url']);
    });

    this.translationService.onTranslationsLoaded.subscribe(res => this.populatePage(this.route.snapshot.params['url']));
  }

  populatePage(url) {
    this.translationService.getHtmlContentByUrl(url).subscribe(res => { 
      this.translation = res.result;
      this.content = this.sanitizer.bypassSecurityTrustHtml(res.result.value) 
      this.changeDetectorRef.detectChanges(); 
      this.titleService.setTitle(this.translation.title);
      this.titleService.setDescription(this.translation.value);
      console.log(this.translation);
      if (this.translation.languageCode != this.translationService.currentLanguage)
        this.translationService.currentLanguage = this.translation.languageCode;
    });
  }

  public get editableContent() {
    return this.translation ? this.sanitizer.bypassSecurityTrustHtml(this.translation.value) : '';
  }
  changeEdit() {
    this.userService.getCurrentUser().subscribe((res) => {
      this.translationService.editContent = res.claims && res.claims['permission'] &&
    res.claims['permission'].indexOf('manageTranslations') >= 0;
    });

}

navigateUrl(url: string) {
  this.zone.run(() => {
    this.router.navigateByUrl(url);
    this.appRef.tick();
  });
}

openEditor() {
    if (!this.translationService.editContent)
      return;
    this.editorOpen=true;
    $('body').css('margin-bottom','365px');
}

closeEditor() {

    this.editorOpen=false;
    $('body').css('margin-bottom','inherit');
}

quickSaveContent() {
  this.translationService.quickSaveHtmlContent(this.translation.name, this.translation.value).subscribe((res: any)=> {
    if (res === true) {
      this.closeEditor();
    } });
}

handleSave(e) {
  if(e.ctrlKey && (e.which == 83)) {
  e.preventDefault();
  this.quickSaveContent();
  }
}


}
