import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { TranslationSharedService } from '../../services/translation-shared/translation-shared.service';
import { of } from 'rxjs';

@Component({
  selector: 'app-international-phone-number',
  templateUrl: './international-phone-number.component.html',
  styleUrls: ['./international-phone-number.component.css']
})
export class InternationalPhoneNumberComponent implements OnInit {

  public countriesList: any[] = [];
  countryFlag: string;
  localizationCountry: any;
  selectedCountryCode: any;
  public phoneNo

  private _phoneNumber: string;
  countriesObs: any;
  @Input()
  public set phoneNumber(value) {
    this._phoneNumber = value;
    this.onPhoneInput(this._phoneNumber);
  }

  public get currentPhoneNumber() {
    let phoneDialCode = this.countriesList.find(ctr => ctr.code == this.selectedCountryCode).dialCode

    if (this.phoneNo && phoneDialCode) {
      return phoneDialCode + this.phoneNo;
    } else {
      return null;
    }
  }

  @Output()
  phoneNumberChange = new EventEmitter(true);
  selectedCountry: any;




  constructor(
    public translationService: TranslationSharedService,

  ) { }

  ngOnInit() {
    this.getCountries();
    this.getFlagEmoji(this.countriesList);


  }

  getCountries() {
    if (this.countriesList.length > 0)
      return of(this.countriesList);
    this.countriesObs = this.translationService.getCountries().share();
    this.countriesObs.subscribe(res => {
      if (!this.selectedCountryCode) {
        this.translationService.getLocalization().subscribe(r => {

          this.countriesList = res.result;
          this.countriesList.forEach(c => c.displayName = `${c.name} ${this.getFlagEmoji(c.code)} (${c.dialCode})`);
          this.localizationCountry = r.result.country

          if (this.localizationCountry)
            this.onSelectionChange(this.localizationCountry);

        });
      }
    });
    return this.countriesObs;
  }


  onSelectionChange(selectedCode): void {
    this.countriesList.forEach(c => c.displayName = `${c.name} ${this.getFlagEmoji(c.code)} (${c.dialCode})`);

    this.selectedCountry = this.countriesList.find(c => c.code == selectedCode)

    this.selectedCountry.displayName = `${this.getFlagEmoji(this.selectedCountry.code)} (${this.selectedCountry.dialCode})`;
    this.selectedCountryCode = this.selectedCountry.code;



  }

  changePhoneNumber(sendEvent = true) {
    if (this._phoneNumber != this.currentPhoneNumber) {
      this._phoneNumber = this.currentPhoneNumber;
      if (sendEvent)
        this.phoneNumberChange.emit(this.currentPhoneNumber);
    }
  }

  onPaste(pastedText) {
    this.onPhoneInput(pastedText.data);
  }
  onPhoneInput(pastedNumber = this.phoneNo) {
    this.getCountries().subscribe(c => {

      this.onSelectionChange(this.selectedCountryCode);

      if (!pastedNumber)
        return;

      console.log(pastedNumber);

      pastedNumber = pastedNumber;

      let matchingCountry = this.countriesList.find(c => {
        return c.dialCode == pastedNumber.substring(0, c.dialCode.length);
      });
      if (matchingCountry) {
        this.phoneNo = pastedNumber.slice(matchingCountry.dialCode.length).trim();
        this.phoneNo = this.phoneNo.replace(/(^0+)|([^0-9+])/g, '');
        this.onSelectionChange(matchingCountry.code);

      } else {
        this.phoneNo = pastedNumber;
        this.phoneNo = this.phoneNo.replace(/(^0+)|([^0-9+])/g, '')
      }

      this.changePhoneNumber();
    });

  }


  getFlagEmoji(countryCodeFlag) {
    if (countryCodeFlag.length < 2)
      return;
    this.countryFlag =
      [...countryCodeFlag?.toUpperCase()]
        .map((char) => String.fromCodePoint(127397 + char.charCodeAt(0)))
        .reduce((a, b) => `${a}${b}`);
    return this.countryFlag

  }


}
