import { Component, OnInit, HostListener, ViewChild, Input, SimpleChanges, HostBinding, TemplateRef, ElementRef } from '@angular/core';
import { FlowHttpClient, GlobalService } from '../../services';
import { Observable, timer } from 'rxjs';
import { DomSanitizer } from '@angular/platform-browser';
import { timingSafeEqual } from 'crypto';

@Component({
  selector: 'a[loading-button],button[loading-button]',
  templateUrl: './loading-button.component.html',
  styleUrls: ['./loading-button.component.css'],
})
export class LoadingButtonComponent implements OnInit {
  public _class: string;
  public _disabled: boolean;
  completeTimer: any;
  errorSubscription: any;
  successSubscription: any;
  elementWidth: any;
  elementHeight: any;

  @Input('class')
  @HostBinding('class')
  public get class() {
    return (this._class ? this._class : ' ') + (this.success == 'SUCCESS' ? ' btn btn-success' : (this.success == 'FAILURE' ? ' btn btn-danger' : (this.color ? ' btn btn-' + this.color : ''))) + (this.showLoading
      ? ' btn-loading ' : '');
  }

  public set class(value) {
    this._class = value;
  }


  constructor(public sanitizer: DomSanitizer, public globalService: GlobalService,
    private elRef: ElementRef
  ) { }

  public showLoading: boolean = false;
  public _success: string;
  public subscription: any;
  public timeout: any;
  public innerSuccess: boolean;

  @Input('disabled')
  @HostBinding('disabled')
  public get disabled() {
    return this._disabled;
  }

  public set disabled(value) {
    this._disabled = value;
  }


  @Input('color')
  public color: string;

  @Input('type')
  public type: string;

  @Input('style')
  public style: string;


  public get success() {
    return this._success;
  }
  public set success(value) {
    this._success = value;
  }


  safecss(input) {
    if (input)
      return this.sanitizer.bypassSecurityTrustStyle(input);
    else
      return '';
  }
  ngOnInit() {
    if (this.elRef.nativeElement.className.includes('btn-info'))
      this.color = 'info'
    if (this.elRef.nativeElement.className.includes('btn-primary'))
      this.color = 'primary'
    if (this.elRef.nativeElement.className.includes('btn-warning'))
      this.color = 'warning'
    if (this.elRef.nativeElement.className.includes('btn-danger') || this.elRef.nativeElement.className.includes('btn-error'))
      this.color = 'danger'
    if (this.elRef.nativeElement.className.includes('btn-default'))
      this.color = 'default'


    this.errorSubscription = this.globalService.onError.subscribe(res => {
      if (this.showLoading) {
        this.success = 'FAILURE';
        this.showLoading = false;
        this.elRef.nativeElement.className = this.elRef.nativeElement.className.replace(`btn-${this.color}`, `btn-error`);
        this.completeTimer = timer(2000).subscribe(res => {
          this.success = null;
          this.disabled = false;
          this.elRef.nativeElement.className = this.elRef.nativeElement.className.replace(`btn-error`, `btn-${this.color}`);
        });
      }
    });

    this.successSubscription = this.globalService.onSuccess.subscribe(res => {
      if (this.showLoading) {
        this.showLoading = false;
        this.success = 'SUCCESS';
        this.elRef.nativeElement.className = this.elRef.nativeElement.className.replace(`btn-${this.color}`, `btn-success`);
        this.completeTimer = timer(2000).subscribe(res => {
          this.success = null;
          this.elRef.nativeElement.className = this.elRef.nativeElement.className.replace(`btn-success`, `btn-${this.color}`);
          this.disabled = false;
        });
      }
    });
  }

  @HostListener('click', ["$event"])
  handleClick(evt) {
    if (this._disabled) {
      evt.stopPropagation();
      return false;
    }
    let style = window.getComputedStyle(this.elRef.nativeElement, null);
    this.elementWidth = this.elRef.nativeElement.clientWidth - parseFloat(style.paddingLeft) - parseFloat(style.paddingRight);
    this.elementHeight = this.elRef.nativeElement.clientHeight - parseFloat(style.paddingTop) - parseFloat(style.paddingBottom);

    if (this.completeTimer)
      this.completeTimer.unsubscribe();
    this.showLoading = true;
    this.disabled = true;

  }

  @HostBinding('style.pointer-events') get pEvents(): string {
    if (this.disabled) {
      return 'none';
    }
    return 'auto';
  }

  ngOnDestroy() {
    if (this.successSubscription)
      this.successSubscription.unsubscribe();

    if (this.errorSubscription)
      this.errorSubscription.unsubscribe();
  }
}
