import { Component, Input, NgZone, OnInit, ViewChild } from '@angular/core';
import { NgForm } from '@angular/forms';
import { Router } from '@angular/router';
import { InAppBrowser } from '@awesome-cordova-plugins/in-app-browser/ngx';
import { environment } from 'environments/environment';
import { FlowHttpClient, GlobalService, ModalService, SidebarService, TranslationSharedService, UserAccountService } from 'flow-concept-shared/services';
import { HttpClient } from '@angular/common/http';
import { TranslationPipe } from 'flow-concept-shared';
import { Platform } from '@ionic/angular';
import { FlowCookieService } from 'flow-concept-shared/services/cookie/cookie.service';
import { CodeInputModule } from 'angular-code-input';
import { CountdownComponent } from 'ngx-countdown';


declare var window;

@Component({
  selector: 'app-login-form-new',
  templateUrl: './login-form-new.component.html',
  styleUrls: ['./login-form-new.component.css']
})
export class LoginFormNewComponent implements OnInit {

  @Input()
  public context: any = {};
  @ViewChild('myForm', { static: false })
  private myForm: NgForm;
  @ViewChild('cd', { static: false }) private countdown: CountdownComponent;

  @Input()
  public allowAccountCreation: boolean = true;
  @Input()
  public tokenType: string = "TEMPORARY";
  @Input()
  public clientId: string;
  @Input()
  public type: string = 'EMAIL';
  public showError: boolean;
  public forgotPasswordResult: string;
  public loginMessage: string;
  public env: any;
  public status: string;
  public cordova: any;
  public loginSubscription: any;
  public accessRefused: boolean;
  public emailSent: boolean;
  public verifyPhone: boolean;
  public provincesList: any[] = [];
  public countriesList: any[] = [];
  loading: boolean;
  otpSuccess: any;
  contextResponse: any;
  responseStatus: any;
  otpError: any;
  otpSent: any;
  verified: any;
  otpVerified: any;
  loginToken: any;
  userProfile: any;
  emailOtp: any;
  phoneOtp: any;
  prevContext: any = {};
  countryFlag: string;
  localizationCountry: any;
  selectedCountryCode: any;
  phoneLoginNumber: any;
  profileCountryCode: any;
  currentCountry: any;
  currentProvince: any;
  showResend: boolean = true;
  public expTimestamp: any;

  public countdownConfig: any = {};



  public get passwordError() {
    if (!this.context.password)
      return 'error.login-form.password';
    if (this.context.password.length < 6)
      return 'error.login-form.password-length';

    return null;
  }

  constructor(public userService: UserAccountService,
    public sidebarService: SidebarService,
    public iab: InAppBrowser,
    public router: Router,
    public zone: NgZone,
    public modalService: ModalService,
    public http: HttpClient,
    public translationService: TranslationSharedService,
    public httpClient: FlowHttpClient,
    public platform: Platform,
    public cookieService: FlowCookieService,
    public globalService: GlobalService
  ) {
  }

  ngOnInit() {
    var result: any = {};

    this.context.type = this.context.type ?? this.type;
    this.context.tokenType = this.context.tokenType ?? this.tokenType;
    this.context.action = this.context.action ?? 'LOGIN';
    this.context.allowAccountCreation = typeof this.context.allowAccountCreation == 'boolean' ? this.context.allowAccountCreation : this.allowAccountCreation;

    this.getCountries();

    this.userService.onAccessRefused.subscribe(res => {
      this.resetContext();
      this.accessRefused = true;
    });

    this.env = environment;
    this.cordova = window.cordova;
    if (!this.userService.redirectTo)
      this.userService.redirectTo = this.context.returnUrl || window.location.pathname + window.location.search;


    (<any>window).login = (<any>window).login || this;

    if (this.context.otpType == 'EMAIL') {
      this.context.otp = this.emailOtp
    }

    if (this.context.otpType == 'PHONE') {
      this.context.otp = this.phoneOtp
    }
  }

  ngOnDestroy() {
    if (this.loginSubscription)
      this.loginSubscription.unsubscribe();

    window.login = null;
  }



  openConsent(evt, item) {
    evt.stopPropagation();
    this.http.get(item.contentUrl, { responseType: 'text' }).subscribe(r => {
      this.modalService.openRawAlert(new TranslationPipe().transform('consentTitle.' + item.name, this.translationService.translations), r);
    });
  }

  recover() {
    this.context.action = 'RECOVER';
    this.context.type = this.context.type == 'EMAIL' ? 'PHONE' : 'EMAIL';
  }

  getOtp() {
    this.showError = false;
    this.context.errorMessage = null;
    // this.showResend = false;


    if (this.myForm && !this.myForm.valid) {
      this.showError = true;
      this.globalService.onError.emit();
      return;
    }

    this.context.clientName = environment.client_id;
    this.context.allowAccountCreation = this.allowAccountCreation;
    this.userService.getOtp(this.context).subscribe(res => {
      this.prevContext = this.context;
      this.context = res.result;
      this.countdownConfig = { leftTime: 30, format: 'ss' };

      if (this.context.status == 'error') {
        this.globalService.onError.emit();
      }

      // if (this.context.type == 'PHONE') {
      //   const input = document.querySelector("#phone");
      //   intlTelInput(input);
      // }

      if (this.context.errorMessage == 'duplicate_phone') {
        this.globalService.onError.emit();
        console.log('duplicate_phone')
        return;
      }


    })
  }


  verifyOtp(val) {
    this.context.errorMessage = null;

    this.context.clientName = environment.client_id;
    this.context.otp = val;
    this.context.status = 'otp_sent';
    this.loading = true;

    this.otpError = false;
    this.otpSuccess = false;

    this.userService.verifyOtp(this.context).subscribe(r => {
      if (r.result.status == 'otp_verified') {
        this.otpSuccess = true;
        this.globalService.onSuccess.emit();
        if (r.result.type=='TOKEN') {
          this.userService.processOtpToken(r.result);
          return;
        }
        if (r.result.action == 'CHANGELOGINNAME' && r.result.status == 'login_name_changed') {
          this.userService.currentUser = r.result.user;
          this.userService.onUserLoggedIn.emit({ action: this.context.action, user: r.result.user });
          return;
        }
  
      }

      this.loading = false;

      this.context = r.result
      console.log(this.context);

      if (this.context.errorMessage == 'wrong_code') {
        this.context.otp = null;
        this.emailOtp = null;
        this.phoneOtp = null;
        this.otpError = true;
      }

      if (this.context.type == 'DATA') {
        this.currentCountry = this.translationService.countries.find(c => c.code == this.context.user.countryCode);
        if (this.currentCountry?.provinces?.length)
          this.currentProvince = this.currentCountry.provinces.find(p => p.code == this.context.user.provinceCode);
      }


    })
  }

  countdownEvent(evt) {
    if (evt.action == 'finished' || evt.action == 'done') {
      // this.showResend = true;
    }
  }

  resendOtp() {
    this.userService.resendOtp(this.context).subscribe(res => {
      
      if (res.result.status == 'error') {
        this.context.errorMessage = res.result.errorMessage;
        this.globalService.onError.emit();
      }
      else {
        this.globalService.onSuccess.emit();
        // this.showResend = false;
        this.context = res.result;
        this.countdownConfig = { leftTime: 30, format: 'ss' };
        
      }
    })
  }

  saveUser() {
    this.showError = false;
    if ((this.myForm && !this.myForm.valid) || this.context.user.consents.find(c => c.required && !c.consented)) {
      this.globalService.onError.emit();
      this.showError = true;
      return;
    }

    this.context.errorMessage = null;

    this.userService.saveUserProfile(this.context).subscribe(r => {
      this.userService.processOtpToken(r.result);
    })
  }
  resetContext() {
    this.prevContext.token = null;
    this.prevContext.loginName = null;
    this.context = this.prevContext;
  }

  resetContextData() {


  }

  submitRegister() {
    this.context.token = this.loginToken;
    this.context.errorMessage = null;

    this.userService.verifyOtp(this.context).subscribe(r => {
      this.contextResponse = r.result
      if (r.result.status == 'error')
        this.globalService.onError.emit();
      else
        this.globalService.onSuccess.emit();
    })
  }


  getCountries() {
    this.translationService.getLocalization().subscribe(r => {
      this.localizationCountry = r.result.country;
      this.profileCountryCode = this.localizationCountry ? this.localizationCountry : this.context.userProfile.countryCode
    });
  }


  selectCountry() {
    this.context.user.country = this.currentCountry?.name;
    this.context.user.countryCode = this.currentCountry?.code;
    this.context.province = null;
    this.context.provinceCode = null;
    this.currentProvince = undefined;
  }


  selectProvince() {
    this.context.user.province = this.currentProvince?.name;
    this.context.user.provinceCode = this.currentProvince?.code;
  }



  remove() {
    this.cookieService.remove('userAccount');
    this.resetContext();
  }



}
