import { Component, Input, NgZone, OnInit, ViewChild } from '@angular/core';
import { NgForm } from '@angular/forms';
import { Router } from '@angular/router';
import { InAppBrowser } from '@awesome-cordova-plugins/in-app-browser/ngx';
import { environment } from 'environments/environment';
import { FlowHttpClient, GlobalService, ModalService, SidebarService, TranslationSharedService, UserAccountService } from 'flow-concept-shared/services';
import { GooglePlus } from '@awesome-cordova-plugins/google-plus/ngx';
import { Facebook } from '@awesome-cordova-plugins/facebook/ngx';
import { HttpClient } from '@angular/common/http';
import { TranslationPipe } from 'flow-concept-shared';
import { Platform } from '@ionic/angular';
import { FlowCookieService } from 'flow-concept-shared/services/cookie/cookie.service';
declare var window;
declare var AppleID;

@Component({
  selector: 'app-login-form',
  templateUrl: './login-form.component.html',
  styleUrls: ['./login-form.component.css']
})
export class LoginFormComponent implements OnInit {

  @Input()
  public context: any = {};
  @ViewChild('myForm', { static: false })
  private myForm: NgForm;
  @Input()
  public allowAccountCreation: boolean = true;
  @Input()
  public clientId: string;

  public showError: boolean;
  public forgotPasswordResult: string;
  public loginMessage: string;
  public env: any;
  public fbPixel: string;
  public status: string;
  public cordova: any;
  public loginSubscription: any;
  public googleLoading: boolean;
  public facebookLoading: boolean;
  public appleLoading: boolean;
  public isApple: boolean
  public accessRefused: boolean;
  emailIsApple: boolean;
  editLocation: boolean;
  public showPassword: boolean = false;

  public get passwordError() {
    if (!this.context.password)
      return 'error.login-form.password';
    if (this.context.password.length < 6)
      return 'error.login-form.password-length';

    return null;
  }

  constructor(public userService: UserAccountService,
    public sidebarService: SidebarService,
    public iab: InAppBrowser,
    public router: Router,
    public zone: NgZone,
    public googlePlus: GooglePlus,
    public fb: Facebook,
    public modalService: ModalService,
    public http: HttpClient,
    public translationService: TranslationSharedService,
    public httpClient: FlowHttpClient,
    public platform: Platform,
    public cookieService: FlowCookieService,
    public globalService: GlobalService
  ) {
  }

  ngOnInit() {
    var result: any = {};

    this.userService.onAccessRefused.subscribe(res => {
      this.resetContext();
      this.accessRefused = true;
    });

    this.env = environment;
    this.fbPixel = this.context.clientFbPixel;
    this.isApple = this.platform.is('ios');
    this.cordova = window.cordova;
    if (!this.userService.redirectTo)
      this.userService.redirectTo = this.context.returnUrl || window.location.pathname + window.location.search;
    if (environment.apple) {
      let result = AppleID.auth.init({
        clientId: environment.apple.clientId,
        scope: 'name email',
        redirectURI: window.location.protocol + '//' + window.location.host + '/',
        usePopup: true, //or false defaults to false
      });
    }

    (<any>window).login = (<any>window).login || this;

    let userString = this.cookieService.get('userAccount');
    if (userString) {
      this.context = JSON.parse(userString);
      this.submitEmail();
    }
  }

  ngOnDestroy() {
    if (this.loginSubscription)
      this.loginSubscription.unsubscribe();

    window.login = null;
  }

  openFacebookLogin() {
    if (window.cordova) {
      this.facebookLoading = true;
      this.fb.login(['public_profile', 'email'])
        .then(res => this.userService.takeExternalLogin(res.authResponse, 'facebook').subscribe(r => {
          this.loginResponse(r);
          this.facebookLoading = false;
        }));
    }
    else {
      window.open(this.userService.getExternalSignInUrl('facebook', this.userService.redirectTo));
      //window.location.href=this.userService.getExternalSignInUrl('facebook', this.userService.redirectTo);
    }
  }

  openConsent(evt, item) {
    evt.stopPropagation();
    this.http.get(item.contentUrl, { responseType: 'text' }).subscribe(r => {
      this.modalService.openRawAlert(new TranslationPipe().transform('consentTitle.' + item.name, this.translationService.translations), r);
    });
  }

  openGoogleLogin() {

    if (window.cordova) {
      this.googleLoading = true;
      this.googlePlus.login({}).catch(err => err).then(res => {
        this.userService.takeExternalLogin(res, 'google').subscribe(r => {
          this.loginResponse(r);
          this.googleLoading = false;
        });
      });
      // var ref = this.iab.create(this.userService.getExternalSignInUrl(liveTicketsSettings.googleProvider, "https://www.livetickets.ro/login"));
      // ref.on('loadstart').subscribe(params => {
      //   if (params.url.indexOf('https://www.livetickets.ro/login') == 0) {
      //     let loc = new URL(params.url);
      //     this.router.navigateByUrl(`/login${loc.hash}`);
      //     ref.close();
      //   }
      // });
    }
    else {
      // if (this.globalService.browser.name.toLowerCase()=='safari') {
      window.open(this.userService.getExternalSignInUrl('google', this.userService.redirectTo));
      // }
      // else {
      //   this.sidebarService.openIframeSidebar(this.userService.getLoginUrl(this.environment.client_id, true, this.userService.redirectTo));
      // }
    }


  }


  forgotPassword() {
    this.userService.forgotPassword(this.context).subscribe((res: any) => {
      this.forgotPasswordResult = res.response == 'reset-password-success' ? 'reset-password-success' : res.error;
    });
  }

  submitEmail() {
    this.showError = false;
    if (this.myForm && !this.myForm.valid) {
      this.globalService.onError.emit();
      this.showError = true;
      return;
    }
    this.context.clientId = this.clientId ?? environment.client_id;

    this.userService.prelogin(this.context).subscribe((res: any) => {
      this.context = res;
      this.status = res.response;
      this.globalService.onSuccess.emit();

      if (res.response == 'not_found') {
        if (this.allowAccountCreation) {
          this.translationService.getLocalization().subscribe(r => {
            this.context.city = r.result.city;
            this.context.countryCode = r.result.country;
            this.context.province = r.result.province;
            this.editLocation = !(this.context.city && this.context.countryCode && this.context.province);

          });
        }
        else {
          this.status = 'creation_denied';
        }

      }
    });

  }

  submitLogin() {
    this.showError = false;
    this.context.response = undefined;
    this.context.error = undefined;
    if (!this.myForm.valid) {
      this.showError = true;
      this.globalService.onError.emit();

      return;
    }
    this.context.clientId = this.clientId ?? environment.client_id;

    this.userService.login(this.context).subscribe((res: any) => {
      this.loginResponse(res);
    });

  }

  loginResponse(res) {
    this.context.response = res.response;
    this.context.error = res.error;
    if (this.context.error) {
      this.globalService.onError.emit();
    }
    else {
      this.globalService.onSuccess.emit();
    }
    if (res.response == 'fill_data') {
      res.firstName = res.userAccount.firstName;
      res.lastName = res.userAccount.lastName;
      res.email = res.userAccount.email;
      res.phone = res.userAccount.phoneNumber;
      res.city = res.userAccount.city;
      res.province = res.userAccount.province;
      res.countryCode = res.userAccount.countryCode;
      var returnUrl = this.context.returnUrl;
      Object.assign(this.context, res);
      this.context.returnUrl = returnUrl;
      this.status = this.context.response;
    }

  }

  submitRegister() {

    this.showError = false;

    
    this.context.clientId = this.clientId ?? environment.client_id;
    if (!this.myForm.valid || this.passwordError || this.context.consents.find(c => c.required && !c.consented)) {
      this.showError = true;
      this.globalService.onError.emit();
      return;
    }
    this.userService.register(this.context).subscribe((res: any) => {
      this.loginResponse(res);

      if (environment.fbPixel)
        (<any>window).dataLayer.push({ event: 'submitFbRegister', pixel_id: environment.fbPixel });

      if (this.fbPixel) {
        (<any>window).dataLayer.push({ event: 'submitFbRegister', pixel_id: this.fbPixel });
      }
    });

  }

  resetContext() {
    this.showError = false;
    this.accessRefused = false;
    this.context = { clientId: this.clientId ?? environment.client_id, returnUrl: this.context.returnUrl };
    this.status = null;
  }


  externalLogin(obj, provider) {
    this.zone.run(() => {
      if (provider == 'google')
        this.googleLoading = true;
      if (provider == 'facebook')
        this.facebookLoading = true;

      this.userService.takeExternalLogin(obj, provider).subscribe(r => {
        this.loginResponse(r);
        this.googleLoading = false;
        this.facebookLoading = false;
      });

    });
  }

  appleSignIn() {
    if (this.cordova) {
      window.cordova.plugins.SignInWithApple.signin(
        { requestedScopes: [0, 1] },
        res => {
          this.zone.run(() => {
            this.appleLoading = true;
            let submitObj: any = {
              authorization: {
                code: res.authorizationCode,
                id_token: res.identityToken
              }
            };
            if (res.fullName.familyName && res.fullName.familyName != '') {
              submitObj.user = {
                name: {
                  firstName: res.fullName.givenName,
                  lastName: res.fullName.familyName
                }
              };
            }

            this.userService.takeExternalLogin(submitObj, 'apple').subscribe(r => {
              this.loginResponse(r);
              this.appleLoading = false;
            });
          });

        },
        err => {

        })
    }
    else {
      AppleID.auth.signIn().then(res => {
        this.appleLoading = true;
        this.userService.takeExternalLogin(res, 'apple').subscribe(r => {
          this.loginResponse(r);
          this.appleLoading = false;
        });
      });
    }
  }

  remove() {
    this.cookieService.remove('userAccount');
    this.resetContext();
  }

  seePassword() {
    this.showPassword = !this.showPassword

  }
}
