import { Component, OnInit, Input, HostBinding, HostListener, ViewChild, ElementRef } from '@angular/core';
import { SignalrNotificationService } from '../../services';
import { TranslationSharedService } from '../../services';
import { animations } from '../../animations/shared.animations';
import { Observable } from 'rxjs/Rx';
import { SortingPagingCriteria } from '../../misc/sorting-paging-criteria';
import { timer } from 'rxjs';

@Component({
  selector: '[notification-dropdown]',
  templateUrl: './notification-dropdown.component.html',
  styleUrls: ['./notification-dropdown.component.css'],
  animations:animations,
})
export class NotificationDropdownComponent implements OnInit {

  public connection: any;
  @HostBinding('class.open') public notificationsOpen: boolean = false;

  public sortingPagingCriteria: SortingPagingCriteria = new SortingPagingCriteria();

  public canClose: boolean;

  @Input() public icon: string = 'fa fa-bell';
  
  @Input() public translation: string;

  @Input() public linkClass: string;

  @HostListener('document:touchend', ['$event'])
  @HostListener('document:click', ['$event'])
  onDocumentClick(targetElementPath) {
    let elementRefInPath = targetElementPath.path ? targetElementPath.path.find(e => e === this._elementRef.nativeElement) : undefined;
    if (!elementRefInPath && this.notificationsOpen && this.canClose) {
      this.openNotifications();
    }
  }      

  constructor(public signalRService: SignalrNotificationService, private translationService: TranslationSharedService, private _elementRef: ElementRef) { }

  ngOnInit() {
    
  }



  openNotifications()
  {
    this.canClose = false;
    this.notificationsOpen = !this.notificationsOpen;
    timer(100).subscribe(r=> this.canClose = true);
  }

}