import { Component, OnInit } from '@angular/core';
import { SignalrNotificationService } from '../../services';
import { TranslationSharedService } from '../../services';
import { animations } from '../../animations/shared.animations';
import { Observable, of, Subscription, timer } from 'rxjs';

@Component({
  selector: 'app-notification-toaster',
  templateUrl: './notification-toaster.component.html',
  styleUrls: ['./notification-toaster.component.css'],
  animations: animations
})
export class NotificationToasterComponent implements OnInit {

  private connection: any;
  public _messages: any[] = [];
  public newMessage: any;
  public obsTimeout: Observable<any>;
  public obsTimer: Subscription;
  constructor(private signalRService: SignalrNotificationService, private translationService: TranslationSharedService) { }

  ngOnInit() {
    this.signalRService.onNotification.subscribe(m=> {
      m.isToasterUnread = true;
      m.isUnread = true;
      if (this.obsTimer) {
        this.obsTimer.unsubscribe();
      }
      this.obsTimeout = timer(5000);
      this.obsTimer = this.obsTimeout.subscribe(() => this.messages.forEach(m=>{
        m.isToasterUnread = false;
      }));
        
      if (this.messages.length>5)
        this.messages[0].isToasterUnread = false;
    });
  }

  public get messages(): any[] {
    return this.signalRService.messages.filter(m=> m && m.isToasterUnread && m.isUnread);
  }

  sendMessage()
  {
    this.connection.invoke('send', this.newMessage);
    this.newMessage='';
  }

  removeNotification(not) {
    not.isUnread=false;
    not.isToasterUnread=false;
  }

  notificationClick(message) {
    this.signalRService.onNotificationClick.emit(message);
    this.removeNotification(message);
  }
}