import { Component, OnInit, Output, EventEmitter, Input } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { SortingPagingCriteria } from 'flow-concept-shared/misc/sorting-paging-criteria';
import { PagerService, PagerConfig } from '../../services';

@Component({
  selector: 'app-pager',
  templateUrl: './pager.component.html',
  styleUrls: ['./pager.component.css']
})
export class PagerComponent implements OnInit {

  @Input()
  public config: PagerConfig = { pageSize: 20, pageNo: 0, totalResults: 0, showPages: 5, totalPages: 0 };
  @Input()
  public instance: string;
  @Input()
  public extraClass: string;
  @Input()
  public showTotalPages: boolean = false;

  @Input()
  public spc: SortingPagingCriteria;


  @Input()
  public disappearOnEmpty: boolean = false;

  public startPage: number;
  public endPage: number;
  public pageArray: number[] = [];


  @Output()
  public onPageChange = new EventEmitter(false);

  constructor(
    public pagerService: PagerService,
    public router: Router,
    public route: ActivatedRoute) {
  }

  ngOnInit() {
    if (!this.instance) {
      throw Error('You need to set the instance name for the pager control ( instance="instance_name" )');
    }
    this.config = this.pagerService.registerInstance(this.instance, this);
    this.route.queryParams.subscribe(res => {
      if (!res['pageNo'])
        return;
        
      this.spc.pageNumber = parseInt(res['pageNo']);
      this.onPageChange.emit(this.spc.pageNumber);
    })
    this.populatePager();
  }

  populatePager() {
    this.pageArray = [];
    this.config.totalPages = Math.ceil(this.config.totalResults / this.config.pageSize);
    this.startPage = this.spc.pageNumber < ((this.config.showPages - 1) / 2) ? 1 : this.spc.pageNumber > this.config.totalPages - ((this.config.showPages + 1) / 2) ? this.config.totalPages - this.config.showPages : this.spc.pageNumber - Math.floor((this.config.showPages - 1) / 2);
    this.endPage = this.spc.pageNumber > this.config.totalPages - ((this.config.showPages + 1) / 2) ? this.config.totalPages : this.spc.pageNumber < ((this.config.showPages - 1) / 2) ? this.config.showPages : this.spc.pageNumber + Math.floor((this.config.showPages + 1) / 2);
    if (this.startPage < 1) this.startPage = 1;
    if (this.endPage > this.config.totalPages) this.endPage = this.config.totalPages;
    for (let i = this.startPage; i <= this.endPage; i++) {
      this.pageArray[this.pageArray.length] = i;
    }
  }

  changePage(pageNumber: number) {
    let params = {pageNo: pageNumber};
    this.router.navigate(    [], 
      {
        relativeTo: this.route,
        queryParams: params, 
        queryParamsHandling: 'merge'
      });
  }
}
