import { Component, OnInit, Input, ViewChild, forwardRef, EventEmitter, Output } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { DomSanitizer } from '@angular/platform-browser';
import { UUID } from 'angular2-uuid';
import { environment } from 'environments/environment';
import { NgForm } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { TranslationPipe } from '../../pipes/translation/translation.pipe';
import { TranslationSharedService } from '../../services/translation-shared/translation-shared.service';
import { GlobalService } from 'flow-concept-shared/services';
import { Misc } from 'flow-concept-shared/misc/misc';


const noop = () => { };

@Component({
  selector: 'app-questions-answers',
  templateUrl: './questions-answers.component.html',
  styleUrls: ['./questions-answers.component.css'],
  providers: [{
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => QuestionsAnswersComponent),
    multi: true
  }]
})
export class QuestionsAnswersComponent {

  private _mode: string = 'answer';
  public showError: boolean;
  @ViewChild('myForm', { static: false })
  private myForm: NgForm;
  public success: boolean;
  public backgroundImage: any;
  public searchString: string;
  public event: any[] = [];
  public eventId: string;

  @Input()
  public startDate: any;
  @Input()
  public adultAge: number;
  
  public autoCheckin: boolean;

  @Input()
  public imageRoot: any;

  @Output()
  public onModelChange: EventEmitter<any> = new EventEmitter(false);
  
  dobAnswer: any;
  firstNameError: boolean;
  lastNameError: boolean;
  textareaError: boolean;
  textboxError: boolean;
  choiceError: boolean;
  dobError: boolean;
  emailError: boolean;
  phoneError: boolean;
  guardianNameError: boolean;
  guardianPhoneError: boolean;


  public get imageItem() {
    if(this.registerQuestions){
    return !this.registerQuestions ? null : this.registerQuestions.find(t => t.type == 'IMAGE');
  }
  }
  public get firstNameItem() {
    return !this.registerQuestions ? null : this.registerQuestions.find(t => t.type == 'FIRSTNAME');
  }
  public get lastNameItem() {
    return !this.registerQuestions ? null : this.registerQuestions.find(t => t.type == 'LASTNAME');
  }
  public get dobItem() {
    return !this.registerQuestions ? null : this.registerQuestions.find(t => t.type == 'DATEOFBIRTH');
  }
  public get extraFields() {
    return !this.registerQuestions ? null : this.registerQuestions.filter(t => t.type != 'IMAGE' && t.type != 'FIRSTNAME' && t.type != 'LASTNAME' && t.type != 'DATEOFBIRTH') || [];
  }

  public get imageUrl() {
    if (!this.imageItem || (!this.imageItem.answer && !this.imageItem.answerContents))
      return null;
    else
      // return this.sanitizer.bypassSecurityTrustStyle(`url(${this.imageRoot}/misc/${!environment.production ? '_debug/' : ''}${this.imageItem.answer})`);
    return this.sanitizer.bypassSecurityTrustStyle(`url('${this.imageItem.answerContents ? 'data:image/webp;base64,' + this.imageItem.answerContents : this.imageItem.answer }')`);

  }



  @Input()
  public get mode() {
    return this._mode;
  }

  public set mode(value) {
    this._mode = value;
  }



  private onTouchedCallback: () => void = noop;
  private onChangeCallback: (_: any) => void = noop;

  get value(): any {
    return this.registerQuestions;
  };

  @Input()
  set value(v: any) {
    if (v !== this.registerQuestions) {
      this.registerQuestions = v;
      this.onChangeCallback(v);
    }
  }

  //Set touched on blur
  onBlur() {
    this.onTouchedCallback();
  }

  //From ControlValueAccessor interface
  writeValue(value: any) {
    if (value !== this.registerQuestions) {
      this.registerQuestions = value;
    }
  }

  //From ControlValueAccessor interface
  registerOnChange(fn: any) {
    this.onChangeCallback = fn;
  }

  //From ControlValueAccessor interface
  registerOnTouched(fn: any) {
    this.onTouchedCallback = fn;
  }

  public has(item, type) {
    return !this.registerQuestions.find(t => t.type == type && t.id != item.id);
  }

  public get orderedRegisterQuestions() {
    return this.registerQuestions?.filter(q => q.type!='IMAGE').sort((a, b) => a.priority > b.priority ? 1 : a.priority == b.priority ? 0 : -1) || [];
  }

  public getChoices(strChoices: string) {
    return strChoices.split('\n');
  }

  public registerQuestions: any[] = [];


  public isAdult: boolean = true;

  constructor(
    public sanitizer: DomSanitizer,
    public route: ActivatedRoute,
    public translationService: TranslationSharedService,
    public globalService: GlobalService,

  ) { }

  ngAfterViewInit() {
  }



  addNew() {
    this.registerQuestions.push({ id: UUID.UUID().toUpperCase().replace('-', ''), type: 'TEXTBOX', priority: (Math.max(...this.registerQuestions.map(q => q.priority)) > 0 ? Math.max(...this.registerQuestions.map(q => q.priority)) : 0) + 1 });
  }

  deleteQuestion(item) {
    this.registerQuestions.splice(this.registerQuestions.indexOf(item), 1);
  }

  async fileAdded(content) {
    if (!this.imageItem)
      return null;
    else{
      let imgSource = await Misc.resizeImage(content, 150, 150, 60);
      this.imageItem.answerContents = imgSource;
    }
    this.formChanged();
      // return this.sanitizer.bypassSecurityTrustStyle(`url(${this.imageRoot}/misc/${!environment.production ? '_debug/' : ''}${this.imageItem.answer})`);

  }

  formChanged() {
    this.onModelChange.emit(this.registerQuestions);
  }




}
