import { Component, OnInit, Input, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { DomSanitizer } from '@angular/platform-browser';
import { UUID } from 'angular2-uuid';
import { environment } from 'environments/environment';

const noop = () => { };
export const QUESTIONS_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => QuestionsComponent),
  multi: true
};

@Component({
  selector: 'questions',
  templateUrl: './questions.component.html',
  styleUrls: ['./questions.component.css'],
  providers: [QUESTIONS_CONTROL_VALUE_ACCESSOR]
})

export class QuestionsComponent implements OnInit {

  private _mode: string = 'answer';
  
  @Input()
  public imageRoot: any;

  public get imageItem() {
    return !this.registerQuestions ? null : this.registerQuestions.find(t=>t.type=='IMAGE');
  }
  public get firstNameItem() {
    return !this.registerQuestions ? null : this.registerQuestions.find(t=>t.type=='FIRSTNAME');
  }
  public get lastNameItem() {
    return !this.registerQuestions ? null : this.registerQuestions.find(t=>t.type=='LASTNAME');
  }
  public get extraFields() {
    return !this.registerQuestions ? null : this.registerQuestions.filter(t=>t.type!='IMAGE' && t.type!='FIRSTNAME' && t.type!='LASTNAME') || [];
  }

  public get imageUrl() {
    if (!this.imageItem)
      return null;
    else
      return this.sanitizer.bypassSecurityTrustStyle(`url(${this.imageRoot}/misc/${!environment.production ? '_debug/' : ''}${this.imageItem.answer})`);
  }

  @Input()
  public get mode()
  {
    return this._mode;
  }

  public set mode(value)
  {
    this._mode = value;
  }



  private onTouchedCallback: () => void = noop;
  private onChangeCallback: (_: any) => void = noop;

  get value(): any {
    return this.registerQuestions;
  };

  @Input()
  set value(v: any) {
    if (v !== this.registerQuestions) {
      this.registerQuestions = v;
      this.onChangeCallback(v);
    }
  }

  //Set touched on blur
  onBlur() {
    this.onTouchedCallback();
  }

  //From ControlValueAccessor interface
  writeValue(value: any) {
    if (value !== this.registerQuestions) {
      this.registerQuestions = value;
    }
  }

  //From ControlValueAccessor interface
  registerOnChange(fn: any) {
    this.onChangeCallback = fn;
  }

  //From ControlValueAccessor interface
  registerOnTouched(fn: any) {
    this.onTouchedCallback = fn;
  }

  public has(item, type) {
    return !this.registerQuestions.find(t=>t.type==type && t.id!=item.id);
  }

  public get orderedRegisterQuestions() {
    return this.registerQuestions.sort((a,b) => a.priority>b.priority ? 1 : a.priority == b.priority ? 0 : -1);
  }

  public registerQuestions: any[] = [];

  constructor(
    public sanitizer: DomSanitizer
  ) { }

  ngOnInit() {

  }

  addNew() {
    this.registerQuestions.push({ id: UUID.UUID(), type: 'TEXTBOX', priority:(Math.max(...this.registerQuestions.map(q => q.priority)) > 0 ? Math.max(...this.registerQuestions.map(q => q.priority)) : 0) + 1 });
    console.log(this.registerQuestions)

  }

  deleteQuestion(item) {
    this.registerQuestions.splice(this.registerQuestions.indexOf(item),1);
    console.log(this.registerQuestions)

  }
}
