import { Component, OnInit, ViewChild, ComponentFactoryResolver, AfterViewInit, ViewContainerRef, Input, TemplateRef } from '@angular/core';
import { SidebarService } from '../../services/';
import { animations } from '../../animations/shared.animations';
import { DomSanitizer } from '@angular/platform-browser';

@Component({
  selector: 'app-sidebar',
  templateUrl: './sidebar.component.html',
  styleUrls: ['./sidebar.component.css'],
  animations: animations
})
export class SidebarComponent implements OnInit, AfterViewInit {

  @ViewChild('content', {read: ViewContainerRef}) content: ViewContainerRef;

  @ViewChild('frame') frame: any;

  public visible: boolean = false;
  public type: string = 'regular';
  public iframeUrl: any;
  public iframeLoading: boolean = true;
  public width = '400';
  @Input()
  loaderTemplate: TemplateRef<any>;

  constructor(private resolver: ComponentFactoryResolver, private sidebarService: SidebarService, private sanitizer: DomSanitizer) { 
    this.iframeLoading = true;
  }

  ngAfterViewInit() {
    this.sidebarService.onSidebarOpen.subscribe(res => {

      this.visible = true;
      this.type=res.type;
      
      this.width = res.width;

      if (res.type=='iframe')
      {
        this.iframeUrl = this.sanitizer.bypassSecurityTrustResourceUrl(res.url);
      }
      else 
      {
        setTimeout(() => {
          let componentFactory = this.resolver.resolveComponentFactory(res.component);
          this.content.clear();
          let componentRef = this.content.createComponent(componentFactory);  
          (<any>componentRef.instance).context = res.data;
        }, 50);
      }
    });
    this.sidebarService.onSidebarClose.subscribe(res => this.close());
  }

  ngOnInit() {
  }
  iframeLoad(evt) {
    this.iframeLoading = false;
    
  }

  closePress() {
    this.sidebarService.closeSidebar();
  }

  close() {
    this.visible = false;
    if (this.content)
      this.content.clear();

  }

}
