import { Component, Input, ApplicationRef, NgZone, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { GlobalService, TranslationSharedService, UserAccountService } from '../../services';
import { TranslationPipe } from '../../pipes';
import { DomSanitizer, SafeHtml } from '@angular/platform-browser';
import { Observable } from 'rxjs';
declare var $;
@Component({
  selector: 'app-translation',
  templateUrl: './translation.component.html',
  styleUrls: ['./translation.component.css']
})
export class TranslationComponent implements OnInit {

  public _name: string;
  @Input('name')
  public set name(value) {
    this._name = value;
    this.ngOnInit();
  }

  public get name() {
    return this._name;
  }
  public _replacements: string[] = [];

  @Input()
  public set replacements(value) {
    this._replacements = value;
    this.ngOnInit();
  }

  public get replacements() {
    return this._replacements;
  }

  public editorOpen: boolean = false;
  public saveSuccess: boolean = false;
  public editContent: boolean = false;
  public content: any;

  public get editableContent() {
    return this.sanitizer.bypassSecurityTrustHtml(new TranslationPipe().transform(this.name, this.translationService.translations));
  }

  constructor(public translationService: TranslationSharedService,
    public router: Router,
    public sanitizer: DomSanitizer,
    public appRef: ApplicationRef,
    public userService: UserAccountService,
    public route: ActivatedRoute,
    public zone: NgZone,
    public globalService: GlobalService
  ) {
    (<any>window).angularContext = (<any>window).angularContext || { parentComponent: this };
    this.translationService.onTranslationsLoaded.subscribe(res => {
      this.ngOnInit();
    });
  }

  ngOnInit() {
    this.content = this.sanitizer.bypassSecurityTrustHtml(this.globalService.formatString(new TranslationPipe().transform(this.name, this.translationService.translations), ...this.replacements));

  }

  changeEdit() {
    this.userService.getCurrentUserv2().subscribe((res) => {
      this.translationService.editContent = res.permissions &&
        res.permissions.indexOf('manageTranslations') >= 0;
    });

  }

  navigateUrl(url: string) {
    this.zone.run(() => {
      this.router.navigateByUrl(url);
      this.appRef.tick();
    });
  }

  openEditor() {
    if (!this.translationService.editContent)
      return;
    this.editorOpen = true;
    $('body').css('margin-bottom', '365px');
  }

  closeEditor() {

    this.editorOpen = false;
    $('body').css('margin-bottom', 'inherit');
  }

  quickSaveContent() {
    this.translationService.quickSaveHtmlContent(this.name, this.translationService.translations[this.name]).subscribe((res: any) => {
      if (res.result === true) {
        this.closeEditor();
      }
    });
  }

  handleSave(e) {
    if (e.ctrlKey && (e.which == 83)) {
      event.preventDefault();
      this.quickSaveContent();
    }
  }
}
