import { Directive, forwardRef, Input } from '@angular/core';
import { AbstractControl, Validator, NG_ASYNC_VALIDATORS } from '@angular/forms';
import { GlobalService } from '../services/';
import { of } from 'rxjs';

@Directive({
  selector: '[appCnpValidation]',
  providers: [
    {
      provide: NG_ASYNC_VALIDATORS,
      useExisting: forwardRef(() => CnpValidationDirective), 
      multi: true
    }
  ]

})
export class CnpValidationDirective implements Validator  {
  @Input('appCnpValidation') model:any;
  
  constructor(private globalService: GlobalService) { }
  validate(c : AbstractControl) {

    if (!this.model || !c.value || c.value=='') {
      return of(null);
    }

    if (c.value && c.value.length<13)
      return of({badFormat: true});
    
    if (this.model.type=='iban')
    {
      return of(!this.globalService.isValidIBAN2(c.value) ? { badFormat: true } : null);
    }
    else
    {
      return of(!this.globalService.checkPID2(c.value) ? { badFormat: true } : null);
    }
  }
 




}