import { Directive, forwardRef, Input } from '@angular/core';
import { AbstractControl, Validator, NG_VALIDATORS, FormGroup } from '@angular/forms';

@Directive({
  selector: '[compare][ngModel]',
  providers: [
    {
      provide: NG_VALIDATORS,
      useExisting: forwardRef(() => CompareDirective), 
      multi: true
    }
  ]
})
export class CompareDirective implements Validator {
  
  private onNotifyValueChange: () => void;


  @Input('compare')
  model:any = {};

  @Input('compareValue')
  set compareValue(value) {
    this.model.value = value;
    let handler = this.onNotifyValueChange;
    if (handler) {
        handler();
    }
    
  }
  
  registerOnValidatorChange(fn: () => void): void {
    this.onNotifyValueChange = fn;
  }
  
  constructor() { 

  }

validate( c : AbstractControl ) {
    let v1 = null;
    let v2 = null;
    let v3 = null;
    if (!this.model.required && (c.value != null))
        return null;

    if (this.model.required && (c.value == null))
        return { compare: false };

    if (this.model.sign == 'typecheck')
    {
        switch(this.model.type) {
            case 'date':
                return !isNaN(Date.parse(c.value)) ? null : { compare: true };
            case 'int':
                return !isNaN(parseInt(c.value)) ? null : { compare: true };
            case 'float':
                return !isNaN(parseFloat(c.value)) ? null : { compare: true };
        }
    }
    switch(this.model.type) {
        case 'date':
            v1 = new Date(c.value);
            v2 = new Date(this.model.value);
            if (this.model.value2)
                v3 = new Date(this.model.value2)
            break;
        case 'number':
            v1 = parseFloat(c.value);
            v2 = parseFloat(this.model.value);
            if (this.model.value2)
                v3 = parseFloat(this.model.value2);
            break;
        case 'string':
            v1 = c.value ? c.value.toString() : null;
            v2 = this.model.value ? this.model.value.toString() : null;
            if (this.model.value2)
                v3 = this.model.value2 ? this.model.value2.toString() : null; 
            break;

    }
        let returnValue=false;
        
        if (!this.model.range) {
        switch(this.model.sign){
            case '<':
                returnValue = v1<v2;
                break;
            case '<=':
                returnValue = v1<=v2;
                break;
            case '=':
                returnValue = v1==v2;
                break;
            case '>=':
                returnValue = v1>=v2;
                break;
            case '>':
                returnValue = v1>v2;
                break;
        }
    }
    else {
        if (this.model.sign == '=')
            returnValue = v2<=v1 && v1<=v3;
        else
            returnValue = v2<v1 && v1<v3;
    }
        if (returnValue)
            return null;
        else
            return { compare: true };
 }

}