import { EventEmitter, Injectable } from '@angular/core';
import * as moment from 'moment';
declare var CKEDITOR: any;
import * as bowser from 'bowser';
import { environment } from 'environments/environment';
import { NavigationStart, Router } from '@angular/router';
declare var $: any;

@Injectable({
	providedIn: 'root'
  })
export class GlobalService {

	public onSuccess: EventEmitter<any> = new EventEmitter(false);
	public onError: EventEmitter<any> = new EventEmitter(false);

  public showLoader: boolean = true;
  public _currentLanguage: string = environment.defaultLanguage;
  public _currentEditLanguage: string = environment.defaultLanguage;

  public datePickerOptions: any = { autoclose:true,forceParse:true,format:'d.mm.yyyy', orientation: 'bottom left', weekStart: 1 };
  public timePickerOptions: any = { showMeridian: false, minuteStep: 15, showSeconds: false, showInputs: false,  explicitMode:true };

	public ckEditorOptions: any;

  public banks: any = {
'NBOR' : 'BANCA NATIONALA A ROMANIEI', 
'ABNA' : 'ABN AMRO BANK ROMANIA', 
'BUCU' : 'ALPHA BANK ROMANIA SA', 
'ARBL' : 'ANGLO ROMANIAN BANK LTD', 
'MIND' : 'ATE BANK ROMANIA SA', 
'BPOS' : 'BANC POST SA', 
'CARP' : 'BANCA COMERCIALA CARPATICA SA', 
'RNCB' : 'BANCA COMERCIALA ROMANA SA', 
'DARO' : 'BANCA CR FIRENZE ROMANIA SA', 
'BROM' : 'BANCA DI ROMA SPA ITALIA - SUCURSALA BUCURESTI', 
'BITR' : 'BANCA ITALO ROMENA SPA', 
'BRDE' : 'BANCA ROMANA PENTRU DEZVOLTARE', 
'BRMA' : 'BANCA ROMANEASCA SA', 
'BTRL' : 'BANCA TRANSILVANIA SA', 
'DAFB' : 'BANK LEUMI ROMANIA SA', 
'MIRB' : 'BLOM BANK EGYPT SAE EGYPT - SUCURSALA ROMANIA', 
'CECE' : 'CASA DE ECONOMII SI CONSEMNATIUNI CEC SA', 
'CITI' : 'CITIBANK ROMANIA SA', 
'CRCO' : 'CREDIT COOP CASA CENTRALA', 
'FNNB' : 'CREDIT EUROPE BANK (ROMANIA) SA', 
'EGNA' : 'EGNATIA BANK (ROMANIA) SA', 
'BSEA' : 'EMPORIKI BANK-ROMANIA SA', 
'EXIM' : 'EXIMBANK ROMANIA', 
'UGBI' : 'GARANTIBANK INTERNATIONAL NV - SUCURSALA ROMANIA', 
'HVBL' : 'HVB BANCA PENTRU LOCUINTE', 
'INGB' : 'ING BANK NV', 
'BREL' : 'LIBRA BANK SA', 
'CRDZ' : 'MKB ROMEXTERRA BANK SA', 
'OTPV' : 'OTP BANK ROMANIA SA', 
'PIRB' : 'PIRAEUS BANK ROMANIA SA', 
'PORL' : 'PORSCHE BANK ROMANIA SA', 
'MIRO' : 'PROCREDIT BANK', 
'RZBL' : 'RAIFFEISEN BANCA PT LOCUINTE SA', 
'RZBR' : 'RAIFFEISEN BANK SA', 
'ROIN' : 'ROMANIAN INTERNATIONAL BANK SA', 
'WBAN' : 'SANPAOLO IMI BANK ROMANIA SA', 
'TRFD' : 'TRANSFOND SA', 
'TREZ' : 'TREZORERIA STATULUI', 
'BACX' : 'UNICREDIT TIRIAC BANK SA', 
'VBBU' : 'VOLKSBANK ROMANIA SA', 
  };
  loading: boolean = true;
  sidebarOpen: boolean;

  constructor() { 
	if (typeof CKEDITOR !== 'undefined')
	this.ckEditorOptions = {
		toolbarGroups: [
			{ name: 'clipboard',   groups: [ 'clipboard', 'undo' ] },
			{ name: 'links' },
			{ name: 'insert', groups: [ 'table', 'horizontalline', 'specialcharacters' ] },
			{ name: 'forms' },
			{ name: 'tools' },
			{ name: 'document',	   groups: [ 'document', 'doctools' ] },
			{ name: 'others' },
			'/',
			{ name: 'basicstyles', groups: [ 'basicstyles', 'cleanup' ] },
			{ name: 'paragraph',   groups: [ 'list', 'indent', 'blocks', 'align', 'bidi' ] },
			{ name: 'colors' },
		],
	
		removeButtons: 'Underline,Subscript,Superscript',
		enterMode: CKEDITOR.ENTER_BR,
		// Set the most common block elements.
	
		allowedContent: true,
		// ALLOW <i></i>
		protectedSource: [...CKEDITOR.config.protectedSource, /<i[^>]*><\/i>/g, /<a[^>]*><\/a>/g],
		// Simplify the dialog windows.
		height: 350,
		ProcessHTMLEntities: false
	};


  }

  getMoment(date) {
    if (date)
      return moment(date).format('YYYY-MM-DD[T]HH:mm');
    else
      return null;
  }

  getDate(date) {
    if (date)
      return moment(date).format('YYYY-MM-DD');
    else
      return null;
  }

  getDateObj(date) {
    if (date)
      return new Date(moment(date).format('YYYY-MM-DD'));
    else
      return null;
  }

  checkPID2( pid )
{
	var pidLength = 13;
	
	// se testeaza lungimea pid-ului
	if( typeof( pid ) != "string" || pid.length != pidLength )
	{   
		return false;
	}

	var controlValues = new Array( 2, 7, 9, 1, 4, 6, 3, 5, 8, 2, 7, 9 );
	var divideValue = 11;

	var sexValue = 0;
	var monthValue = 0;
	var yearValue = 0;
	var dayValue = 0;

	// se citeste valorea referitoare la sex si data de nastere
	try
	{	sexValue = parseInt( pid.substring( 0, 1 ), 10 );

		yearValue = parseInt( pid.substring( 1, 3 ), 10 );
		if( sexValue == 1 || sexValue == 2 || sexValue > 6 )
			yearValue = 1900 + yearValue;
		else if( sexValue == 3 || sexValue == 4 )
			yearValue = 1800 + yearValue;
		else if( sexValue == 5 || sexValue == 6 )
			yearValue = 2000 + yearValue;

		monthValue = parseInt( pid.substring( 3, 5 ), 10 );
		if( monthValue < 1 && monthValue > 12 )
		{	
			return false;
		}

		dayValue = parseInt( pid.substring( 5, 7 ), 10 );
		if( dayValue < 0 && dayValue > 31 )
		{	
			return false;
		}

		if( !this.checkDateInFutureForPID( pid ) )
		{	
		return false;
		}
	}
	catch( e )
	{	
		return false;
	}

	// se testeaza daca sexul e definit corect
	if( sexValue == 0 )
	{	
		return false;
	}

	// aleg secolul
	var centuryValue = 19;
	if( sexValue > 2 && sexValue < 5 )
	{	centuryValue = 18;
	}
	else if( sexValue > 4  )
	{	centuryValue = 20;
	}

	if(yearValue < 100)yearValue += centuryValue * 100;

	// definesc un vector cu numarul de zile ale fiecarei luni din anul yearValue
	var daysInMonth = new Array( 31,
						  28 + ( ( ( yearValue % 4 == 0 && yearValue % 100 != 0 ) || yearValue % 400 == 0 )? 1 : 0 ),
						  31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );

	// se testeaza daca data nasterii este corecta
	if( monthValue * dayValue == 0 ||
		monthValue > 12 ||
		daysInMonth[monthValue -1] < dayValue )
	{	
		return false;
	}


	// se testeaza cifra de control
	var controlValue = 0;
	for( var ind = 0; ind < controlValues.length; ind ++ )
	{	var nVal = parseInt( pid.charAt( ind ), 10 );
		if( isNaN( nVal ) )
		{	
			return false;
		}

		controlValue += nVal * controlValues[ ind ];
	}
	var resultValue = controlValue % divideValue;
	if( resultValue == 10 )
		resultValue = 1;

	if( resultValue != parseInt( pid.charAt( pidLength - 1 ), 10 ) )
	{	
		return false;
	}
	return true;
}


checkDateInFutureForPID( pid )
{  	var crtDate = new Date();

	var sexValue = parseInt( pid.substring( 0, 1 ), 10 );
	var yearValue = parseInt( pid.substring( 1, 3 ), 10 );
	if( sexValue == 1 || sexValue == 2 || sexValue > 6 )
		yearValue = 1900 + yearValue;
	else if( sexValue == 3 || sexValue == 4 )
		yearValue = 1800 + yearValue;
	else if( sexValue == 5 || sexValue == 6 )
		yearValue = 2000 + yearValue;
    var monthValue = parseInt( pid.substring( 3, 5 ), 10 );
    var dayValue = parseInt( pid.substring( 5, 7 ), 10 );

	var crtYear = crtDate.getFullYear();
	if(crtYear < 1900) crtYear += 1900;
	if( yearValue > crtYear )
	{	
		return false;
	}
	else if( yearValue == crtDate.getFullYear() )
	{	if( monthValue > crtDate.getMonth() + 1 )
		{	
			return false;
		}
		else if( monthValue == crtDate.getMonth() + 1 )
		{	if( dayValue > crtDate.getDate() )
			{	
				return false;
			}
		}
	}

	return true;
}







     //definitii de functii apelate
isValidIBAN2(iban) 
{
	/*
	Pasul 1: Se muta primele patru caractere (simbolizand codul de tara si caracterele de verificare) la dreapta codului IBAN. 
Exemplu: RO49AAAA1B31007593840000 devine AAAA1B31007593840000RO49. 

Pasul 2: Se face conversia literelor in numere, conform tabelei de conversie cuprinsa in Anexa nr. 2. 
Exemplu: AAAA1B31007593840000RO49 devine 1010101011131007593840000272449 

Pasul 3: Se aplica algoritmul MOD 97-10 (conform ISO 7064). Pentru ca cele 2 caractere de verificare sa fie corecte (codul IBAN sa fie corect), restul impartirii numarului astfel obtinut la 97 trebuie sa fie 1. 
Exemplu: Restul impartirii lui 1010101011131007593840000272449 la 97 este 1. 
	*/   
	if (!iban)
		return false;
        
    //iban ="RO23TREZ1315069XXX000497";
	iban = iban.toUpperCase( );

	iban = iban.substring(4) + iban.substring(0, 4);	
	iban = this.convert2N(iban);
	
	var mod97 = this.GetRemainder(String(iban), "97");   
    
	var valid = mod97 == 1 ? true : false;
        if( !valid )
        {
            
         	return false;
        }  
    return true;

}

convert2N(iban)
{
	var conversionArray = new Array("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"); 
	var caracter = "";
	var newiban = "";
	var flag = 0;	
	for (var i = 0; i < 24; i++)
	{
		caracter = iban.substring(i, i+1);	
		flag = 0;
		for (var j = 0; j < 26; j++)
		{
			if (caracter == conversionArray[j])
			{
				newiban = newiban + (j+10);
				j = 26;
				flag = 1;	
			}
		}
		if (flag == 0)
			newiban = newiban + caracter;
	}
	return newiban;
}

GetRemainder(x, y) 
{
	var nr = x;
	var len = nr.length;
	var number = parseInt(y,10);
	var remainder = 0;
	var ignore = 0;
	var k = '';

	while (ignore < len) 
	{
		if (remainder == 0) 
		{
			var w1 = y+' ';
			var w2 = w1.length-1;
			k = nr.slice(ignore, ignore+w2);

			ignore += w2;
			remainder = parseInt(remainder+k, 10) % number;
		} 
		else 
		{
			k = nr.slice(ignore, ignore+1);
			ignore += 1;
			remainder = parseInt(remainder+k, 10) % number;
		}
	}

	return remainder;
}

public get browser() {
	return bowser.getParser(window.navigator.userAgent).getBrowser();
}
public hexToRgbA(hex, a) {
	var c;
    if(/^#([A-Fa-f0-9]{3}){1,2}$/.test(hex)){
        c= hex.substring(1).split('');
        if(c.length== 3){
            c= [c[0], c[0], c[1], c[1], c[2], c[2]];
        }
        c= '0x'+c.join('');
        return 'rgba('+[(c>>16)&255, (c>>8)&255, c&255].join(',')+',' + a + ')';
    }
    throw new Error('Bad Hex');
}
formatString(str: string, ...val: string[]) {
	for (let index = 0; index < val.length; index++) {
		str = str.replace(`{${index}}`, val[index]);
	}
	return str;
}

replacer(key, value) {
    // Filtering out properties
    if (typeof value === 'string') {
      return encodeURIComponent(value);
    }
    return value;
  }

  reviver(key, value) {
    // Filtering out properties
    if (typeof value === 'string') {
      return decodeURIComponent(value);
    }
    return value;
  }

  stripHtml(originalString) {
  	let strippedString = originalString ? originalString.replace(/(<([^>]+)>)/gi, '') : '';
	return strippedString;
  }

  encodeBits(array: any[]){
	let encoded = 0;
	for (let i = 0; i < array.length; i++)
		if (array[i])
			encoded |= 1 << i; // Set the corresponding bit

	return encoded;
  }

  decodeBits(encoded: number, length: number){
	let decoded: boolean[] = [];
	for (let i = 0; i < length; i++)
		decoded[i] = (encoded & 1 << i) != 0; // Check if the bit is set
	return decoded;
  }
}
