import { Injectable, EventEmitter } from '@angular/core';
import { UserAccountService, FlowHttpClient, TranslationSharedService, GlobalService } from '../../services';
import { ApplicationConfig } from '../../applicationConfig';
import { SortingPagingCriteria } from '../../misc/sorting-paging-criteria';
import { ReplaySubject, Subject, timer } from 'rxjs';
import { Platform } from '@ionic/angular';
import { environment } from 'environments/environment';
import * as signalR from '@microsoft/signalr';  
import { FlowCookieService } from 'flow-concept-shared/services/cookie/cookie.service';
declare var window: any;
import { ActivatedRoute, Router, NavigationEnd } from '@angular/router';
// import { Geolocation } from '@awesome-cordova-plugins/geolocation/ngx';
import * as bowser from 'bowser';
import { UUID } from "angular2-uuid";


@Injectable({
    providedIn: 'root'
  })
export class SignalrNotificationService {

    private connection: any;
    public messages: any[] = [];
    public newMessage: any;
    public subscribedChannels: any[] = [ ];
    public onNotification = new EventEmitter(true);
    public connectionPromise: Promise<any>;
    public unreadMessageCount: number = 0;
    public hasMorePages: boolean = false;
    public subscription: any;
    public onlineStatus: string = 'OFFLINE';
    statusSubscription: any;
    public interval: any;
    public onConnected: EventEmitter<any> = new EventEmitter(true);
    public onNotificationClick = new EventEmitter(true);
    coordinates: any;
    simInfo: any;
    public get instanceId() {
        let result = this.cookieService.get('instanceId');
        if (!result){
            result = UUID.UUID()
            this.cookieService.put('instanceId', result);
        }
        return result;
    }

    public get mute(){
        return this.cookieService.get('mute')=='true';
    };
    public set mute(value){
        this.cookieService.put('mute', value ? 'true' : 'false');
    };
    public notificationArray = [ 
        {name: 'pop', sound: null},
        {name: 'marimba1', sound: null},
    ];
    constructor(
        private cookieService: FlowCookieService, 
        private userService: UserAccountService, 
        private httpClient: FlowHttpClient, 
        private platform: Platform,
        // private geolocation: Geolocation,
        private activatedRoute: ActivatedRoute,
        public router: Router,
        public http: FlowHttpClient,
        public translationService: TranslationSharedService,
        public globalService: GlobalService,
        ) { 
        this.getNotifications();
        this.getConnection();
        this.preloadSounds();

        
        console.log('subscribing');
        this.platform.resume.subscribe(() => {
            console.log('resuming');
            this.getConnection();
            this.subscribePushChannels();
        });
        
        this.userService.onUserLoggedIn.subscribe(async (res) => {
            this.getNotifications();
            await this.stopConnection();
            await this.getConnection();
           this.subscribePushChannels();
        });

        this.userService.onUserLoggedOut.subscribe(res => {
            this.unsubscribePushChannels(res);
        });

        this.platform.ready().then(p => {
            if (!window.cordova || !window.FirebasePlugin)
                return;

            window.FirebasePlugin.hasPermission((has) => {
                if (has) 
                    { this.initPushNotifications(); }
                else 
                    { window.FirebasePlugin.grantPermission(r=> this.initPushNotifications()); }
            })                
        });
    }



    initPushNotifications() {
        window.FirebasePlugin.getToken();
        window.FirebasePlugin.onMessageReceived(res => {
            console.log(res);
        });
        var channel  = {
            id: "default",
            sound: "correct"
        };
        window.FirebasePlugin.createChannel(channel);
    }

    subscribePushChannels() {
        if (this.userService.currentUser.id && window.cordova && window.FirebasePlugin)
            window.FirebasePlugin.subscribe(`${environment.application_id}${environment.production ? '' : '--debug'}--${environment.client_id.replace('::', '~')}--${this.userService.currentUser.id}`);
    }

    unsubscribePushChannels(user) {
        if (user.id && window.cordova && window.FirebasePlugin)
        window.FirebasePlugin.unsubscribe(`${environment.application_id}${environment.production ? '' : '--debug'}--${environment.client_id.replace('::', '~')}--${user.id}`);
    }
    

    async getConnection() {
        console.log('Getting SignalR connection')
        console.log(this.onlineStatus);
        if (this.interval) this.interval.unsubscribe();

        if(this.onlineStatus == 'ONLINE')
            return;

            // let options = 
            // { 
            //     headers: {
            //         'Authorization': `Bearer ${this.httpClient.token}`
            //     }
            // };
        this.connection = new signalR.HubConnectionBuilder()  
            .configureLogging(signalR.LogLevel.None)
            .withUrl(`${environment.endpoints['NotificationServer']}/SignalRNotificationHub?auth_token=${this.httpClient.token}`)
            .build();
        

        try {
            await this.connection.start();
            this.onlineStatus='ONLINE';
        }
        catch { 
            this.onlineStatus='OFFLINE';
            console.log({ onlineStatus: this.onlineStatus });
            this.interval = timer(5000).subscribe(async() => await this.getConnection());
            return;
        }

        console.log({ onlineStatus: this.onlineStatus });

        this.connection.onclose(async() => {
            this.onlineStatus='OFFLINE';
            console.log({ onClose: this.onlineStatus });
            this.interval = timer(5000).subscribe(async() => await this.getConnection()); 
        });

        // this.connection.onreconnected(() =>{
        //     this.onlineStatus='ONLINE';
        //     console.log({ onReconnect: this.onlineStatus });
        // });

        if (this.onlineStatus == 'ONLINE') {
            if (this.statusSubscription) this.statusSubscription.unsubscribe();
    
            await this.sendCurrentStatus();
            this.statusSubscription = this.router.events.subscribe(async (ev) => {
                if (ev instanceof NavigationEnd)
                    await this.sendCurrentStatus();
                });
        }
        if (this.subscription) this.subscription.unsubscribe();

        this.subscription = this.connection.on('messageReceived', (msg, u) => {
            let m = JSON.parse(msg);
            console.log(m);
            this.onNotification.emit(m);

            if(m.hideInToaster || this.mute)
                return;

            this.messages.push(m);
            this.unreadMessageCount++;
            this.playSound(m.sound);
        });

        console.log('subscribed');
    }

    processReplacements(text: string, strObj: string, prefix: string = 'model') {
    
    let obj = JSON.parse(strObj)

    let replacements = this.buildReplacementsArray(obj, prefix);
    replacements.forEach(element => {
      text = text.replace(`{{${element.query}}}`, element.value);
    });
    return text;

  }
  preloadSounds() {
    this.notificationArray.forEach(s=> {
    s.sound = new Audio();
    s.sound.src = `../assets/audio/${s.name}.mp3`;
    s.sound.load();
  });
  }

  async stopConnection() {
    console.log('stopping connection');
    this.onlineStatus='OFFLINE';

    if(this.interval && this.interval.unsubscribe) {
        this.interval.unsubscribe();
        this.interval = null;
    }

    if (this.connection)
      await this.connection.stop();

  }

  playSound(name) {
      if (!this.mute && name)
      {
        let snd = this.notificationArray.find(n=>n.name == name);
        if (snd && snd.sound)
            snd.sound.play();
      }

  }

  buildReplacementsArray(obj: any, prefix: string = 'model') {
            var str = [], p;
        for (p in obj) {
            if (obj.hasOwnProperty(p)) {
                var k = prefix && !isNaN(p) ? prefix + "[" + p + "]" : prefix && isNaN(p) ? prefix + "." + p : p, v = obj[p];
                str.push((v !== null && typeof v === "object") ?
                    this.buildReplacementsArray(v, k) :
                    { query: k, value: v});
            }
        }
        return str;

  }

  

  public messagePageNo = 1;


  public get browser() {
    return bowser.getParser(window.navigator.userAgent);
    // console.log(this.browser.getBrowser().name);


}

// async updateDeviceStatus() {

//     this.coordinates = await this.geolocation.getCurrentPosition({ timeout: 30000 }).catch(err => {
//         console.log(err);
//         throw err;
//     });
// }


async sendCurrentStatus() {
    await this.userService.getCurrentUser().toPromise();
    let packageInfo=require('/package.json');
    let objSettings = {
        ApplicationId: environment.application_id,
        ClientId: environment.client_id,
        BrowserName: this.browser.getBrowser().name, // "Microsoft Edge"
        BrowserVersion: this.browser.getBrowser().version, //114.0.1823.79
        Route: window.location.pathname + window.location.search, // /access
        Host: window.location.host,
        User: this.userService.currentUser.id ? this.userService.currentUser : null, //{roles: Array(0)}
        Id: this.instanceId, //"372d46d4-a48a-2ca9-c29a-acdeee6798ef"
        Language: this.translationService.currentLanguage, // desktop
        // Lat: this.coordinates ? this.coordinates.coords.latitude : null, //null
        // Lng: this.coordinates ? this.coordinates.coords.longitutde : null, //null
        NetworkOperator: this.simInfo ? this.simInfo.carrierName : null,
        OsName: this.browser.getOS().name, //Windows
        OsVersion: this.browser.getOS().version, //NT 10.0
        PlatformType: this.browser.getPlatformType(),
        ScreenH: this.platform.height(),    //721  
        ScreenW: this.platform.width(), //1026
        Version: packageInfo.version,
    
    };
    await this.connection.send('SendStatus', objSettings);
}

async subscribe(channelId) {
    await this.connection.send('Subscribe', channelId, environment.client_id);
    console.log(`subscribed to ${environment.client_id}::${channelId}`)
}

//offline?





  getNotifications() {
      let spc = new SortingPagingCriteria();
      spc.pageNumber = this.messagePageNo;
      let obs = this.httpClient.get(environment.endpoints['NotificationServer'], `/api/services/app/NotificationService/GetNotifications?${this.httpClient.serializeToQueryString(spc, 'spc')}`, null, false);
      obs.subscribe(res => {
            if (res.result.type!=='error') {
                if (this.messagePageNo == 1)
                {
                    this.messages = this.messages.filter(m=>!m.saveToDb);
                }
                this.messages = this.messages.concat(res.result.notifications);
                this.unreadMessageCount = res.result.unreadMessageCount;
                this.hasMorePages = res.result.hasMorePages;
            }
            
      });


      return obs;
  }

  setReadReceipt() {
      this.unreadMessageCount = 0;
      return this.httpClient.post(environment.endpoints['NotificationServer'], '/api/services/app/NotificationService/SetReadReceipt', null);
  }

  

}


