import { DOCUMENT } from '@angular/common';
import { Inject, Injectable, Renderer2, RendererFactory2 } from '@angular/core';
import { Meta, Title } from '@angular/platform-browser';
import { NavigationStart, Router } from '@angular/router';
import { GlobalService, SignalrNotificationService, TranslationPipe, TranslationSharedService } from 'flow-concept-shared';

@Injectable()
export class TitleService {
    
    private _title: string;
    private renderer2: Renderer2;

constructor(
    public titleService: Title,
    private meta: Meta,
    public globalService: GlobalService,
    public rendererFactory2: RendererFactory2,
    @Inject(DOCUMENT) private _document,
    public router: Router,

) { 
    this.renderer2 = rendererFactory2.createRenderer(null, null);
    this.router.events.subscribe(e => 
        {
            if (e instanceof NavigationStart){
                if (this._document.getElementById('ldJson'))
                    this._document.getElementById('ldJson').remove();
                if (this._document.getElementById('relCanonical'))
                    this._document.getElementById('relCanonical').remove();

            }
        })
}

public setTitle(title = null) {
    let processedTitle = this.globalService.stripHtml(title);
    processedTitle = processedTitle.substring(0, Math.min(processedTitle.length, 60));

    this.titleService.setTitle(processedTitle);
    this.meta.updateTag({ property: 'og:title', content: processedTitle });
}

setDescription(description) {
    let processedDescription = this.globalService.stripHtml(description);
    processedDescription = processedDescription.substring(0, Math.min(processedDescription.length, 150));
    this.meta.updateTag({ property: 'og:description', content: processedDescription});
    this.meta.updateTag({ name: 'description', content: processedDescription});

}

setLdJson(schemaOrg) {
    let ldJson = this.renderer2.createElement('script');
    ldJson.type = 'application/ld+json';
    ldJson.id='ldJson';
    ldJson.text = JSON.stringify(schemaOrg);
    this.renderer2.appendChild(this._document.head, ldJson);
}

setUrl(relativeUrl = window.location.pathname + window.location.search) {
    let relCanonical = this.renderer2.createElement('link');
    relCanonical.rel='canonical';
    relCanonical.id = 'relCanonical';
    relCanonical.href=`${window.location.protocol}//${window.location.host}${relativeUrl}`;
    this.renderer2.appendChild(this._document.head, relCanonical);
    this.meta.updateTag({ property: 'og:url', content: `${window.location.protocol}//${window.location.host}${relativeUrl}` });
}
setImage(image) {
    this.meta.updateTag({ property: 'og:image', content: image });
}

setHttpStatus(status, content = null) {
    this.meta.updateTag({ name: 'prerender-status-code', content: status.toString() });
    if (content)
        this.meta.updateTag({ name: 'prerender-header', content: content });
}

}