import { Injectable, EventEmitter } from '@angular/core';
import { FlowHttpClient } from '../httpClient/httpClient';
import { ApplicationConfig } from '../../applicationConfig';
import { ApplicationSharedService } from '../application-shared/application-shared.service';
import { GlobalService } from '../global/global.service';
import { of } from 'rxjs';
import { FlowCookieService } from 'flow-concept-shared/services/cookie/cookie.service';
import { HttpErrorResponse } from '@angular/common/http';
import { environment } from 'environments/environment';
import { UUID } from 'angular2-uuid';
import { catchError } from 'rxjs/operators';

@Injectable({
  providedIn: 'root'
})
export class TranslationSharedService {

  public translationsLoaded: boolean = false;
  public onLanguageChanged = new EventEmitter(true);
  public onEditLanguageChanged = new EventEmitter(true);
  public onTranslationsLoaded = new EventEmitter(true);

  public translations: any = {};

  public languages: any[] = [];

  public countries: any[] = [];


  public editContent: boolean = false;
  private translationObs: any;
  private countryObs: any;

  public instanceId: any;

  public _currentEditLanguage: string = environment.defaultLanguage;
  public _currentItemLanguage: string = environment.defaultLanguage;

  public get endpoint() {
    return environment.endpoints['TranslationServer'];
  }


  public get currentLanguage() {
    let value = this.cookieService.get('language');
    if (value)
      this.globalService._currentLanguage = value;
    return this.globalService._currentLanguage;
  }
  public set currentLanguage(value) {
    if (!value)
      return;

    let languageChanged = false;
    if (this.globalService._currentLanguage != value)
      languageChanged = true;
    this.globalService._currentLanguage = value;
    this.cookieService.put('language', value);
    if (languageChanged) {
      this.onLanguageChanged.emit(value);
      this.getTranslations(true);
    }

  }

  public get currentEditLanguage() {
    let value = this.cookieService.get('edit-language');
    if (value)
      this.globalService._currentEditLanguage = value;
    return this.globalService._currentEditLanguage;
  }
  public set currentEditLanguage(value) {
    if (!value)
      return;

    this.http.recordPageLoading = true;
    let languageChanged = false;
    if (this.globalService._currentEditLanguage != value)
      languageChanged = true;
    this.globalService._currentEditLanguage = value;
    this.cookieService.put('edit-language', value);
    if (languageChanged) {
      this.onEditLanguageChanged.emit(value);
    }


  }


  public get currentLanguageObj() {
    if (this.languages && this.languages[this.currentLanguage])
      return this.languages[this.currentLanguage];
    else
      return {};
  }

  constructor(public http: FlowHttpClient, public applicationService: ApplicationSharedService, public globalService: GlobalService, public cookieService: FlowCookieService) {
    this.instanceId = UUID.UUID();
  }

  getTranslations(force = true): any {
    if (this.translationsLoaded && !force)
      return of(this.translations);
    if (this.translationObs && !force)
      return this.translationObs;

    if (localStorage['translations']) {
      try {
        this.translations = JSON.parse(localStorage['translations']);
      } catch { }
      this.translationsLoaded = true;
      //this.onTranslationsLoaded.emit();
    }

    this.translationObs = this.http.get(this.endpoint, `/api/services/app/TranslationClientService/GetTranslations?applicationId=${encodeURIComponent(environment.application_id)}&languageCode=${this.currentLanguage}&groupId=${environment.groupId}`, null, false)
      .pipe(catchError(err => {
        console.log('Error loading translations');
        return of([]);
      }))
      .share();
    this.translationObs.subscribe(res => {

      this.translations = res.result;
      this.translationsLoaded = true;
      this.onTranslationsLoaded.emit();
      localStorage['translations'] = JSON.stringify(this.translations);
      this.translationObs = null;

    });
    return this.translationObs;
  }
  getLanguages(application_id) {
    let obs = this.http.get(this.endpoint, `/api/services/app/LanguageClientService/GetLanguages?applicationId=${environment.application_id}`, null, false)
      .pipe(catchError(err => {
        console.log('Error loading translations');
        return of([]);
      }))
      .share();
    obs.subscribe(res => {
      if (!(res instanceof HttpErrorResponse))
        this.languages = res.result;
    });
    return obs;
  }


  quickSaveHtmlContent(name: string, content: string) {
    let obs = this.http.post(this.endpoint, `/api/services/app/TranslationAdminService/QuickSave`, { name: name, value: content, languageCode: this.currentLanguage, groupName: environment.groupId })
      .share();
    return obs;
  }

  getCountries() {
    if (this.countryObs) return this.countryObs;

    this.countryObs = this.http.get(this.endpoint, `/api/services/app/CountryService/GetCountries`)
      .pipe(catchError(err => {
        console.log('Error loading translations');
        return of({});
      })).share();
    this.countryObs.subscribe(res => this.countries = res.result);
    return this.countryObs;
  }

  getCountry(code) {
    return this.countries.filter(c => c.code == code)[0];
  }

  getHtmlContentByUrl(url) {
    return this.http.get(this.endpoint, `/api/services/app/TranslationClientService/GetContentPage?applicationId=${encodeURIComponent(environment.application_id)}&url=${encodeURIComponent(url)}&languageCode=${this.currentLanguage}`);
  }

  getLocalization() {
    if (this.cookieService.get('localization'))
      return of(JSON.parse(this.cookieService.get('localization')));
    else {
      let obs = this.http.get(this.endpoint, `/api/services/app/LocalizationService/GetWithClientIp`, null, false);

      obs.subscribe(res => {
        this.cookieService.put('localization', JSON.stringify(res));
      });
      return obs;
    }
  }
}
