import { Injectable, EventEmitter, HostListener } from '@angular/core';
import { FlowHttpClient } from '../httpClient/httpClient';
import { TranslationSharedService } from '../translation-shared/translation-shared.service';
import { of } from 'rxjs';
import { HttpParams } from '@angular/common/http';
import { ActivatedRoute, GuardsCheckEnd, NavigationStart } from '@angular/router';
import { Router } from '@angular/router';
import { Location } from '@angular/common';
import { ApplicationConfig } from '../../applicationConfig';
import 'rxjs/Rx';
declare var window: any;
import { JwtHelperService } from '@auth0/angular-jwt';
import { ModalService } from '../modal/modal.service';
import { SidebarService } from '../sidebar/sidebar.service';
import { LoginFormComponent } from 'flow-concept-shared/components';
import { environment } from 'environments/environment';
import { HttpHeaders } from '@angular/common/http';
import { FlowCookieService } from 'flow-concept-shared/services/cookie/cookie.service';
import { map } from 'rxjs/operators';
import { timer } from 'rxjs';
import * as bowser from 'bowser';
declare var moment: any;


@Injectable({
  providedIn: 'root'
})
export class UserAccountService {

  public currentUser: any = { claims: [] };
  public _redirectTo: string;
  source: any;
  logoutTimer: any;
  public get browser() {
    return bowser.getParser(window.navigator.userAgent);
  }
  public get redirectTo() {
    if (!this._redirectTo)
      this._redirectTo = this.cookieService.get('redirectTo');

    return this._redirectTo;
  }

  public set redirectTo(value) {
    this._redirectTo = value;
    if (!value)
      this.cookieService.remove('redirectTo');
    else
      this.cookieService.put('redirectTo', value);
  }

  // public set loginUser(userAccount){
  //   this.cookieService.put('userAccount', value);

  // }

  public authError: string;
  public onUserLoggedIn = new EventEmitter(false);
  public onUserLoggedOut = new EventEmitter(false);
  public onAccessRefused = new EventEmitter(false);
  public lastRouteAddress: string;
  public accessRefused: boolean;
  // public timer: TimerDirective;

  public get token() {
    return this.cookieService.get('auth_token');
  }

  @HostListener('window:focus', ['$event'])
  onFocus(event: any): void {
    this.setLogOutTimer();
  }

  constructor(public cookieService: FlowCookieService, public translationService: TranslationSharedService, public http: FlowHttpClient, public router: Router, public location: Location, public route: ActivatedRoute, public modalService: ModalService, public sidebarService: SidebarService,

  ) {
    this.router.events.subscribe((res: any) => {
      if (res instanceof NavigationStart) {
        this.lastRouteAddress = res.url;
      }
    });

    this.route.queryParams.subscribe(res => {
      if (res['tkk'])
        this.processLoginToken(res['tkk'])
    })
  }

  currentUserObs: any;

  getCurrentUser(force = false) {
    if (this.currentUserObs)
      return this.currentUserObs;
    if ((this.http.token && !this.currentUser.id) || force) {
      let jwtHelper = new JwtHelperService();

      this.currentUserObs = this.getCurrentUserDetails(force).share();
      this.currentUserObs.subscribe(res => {
        if (res)
          this.currentUser = res;
        this.currentUserObs = null;
        if (this.currentUser.id)
          this.setLogOutTimer();
      });

      return this.currentUserObs;
    }
    else {
      return of(this.currentUser);
    }

  }

  getCurrentUserv2(force = false) {
    if (this.currentUserObs)
      return this.currentUserObs;
    if ((this.http.token && !this.currentUser.id) || force) {

      this.currentUserObs = this.getCurrentUserDetailsv2(force).share();
      this.currentUserObs.subscribe(res => {
        if (res.result.status == 'valid')
          this.currentUser = res.result.user;
        else
          this.logout();
        this.currentUserObs = null;
        if (this.currentUser.id)
          this.setLogOutTimer();
      });

      return this.currentUserObs;
    }
    else {
      return of(this.currentUser);
    }

  }

  prelogin(model) {
    return this.http.post(environment.endpoints['IdentityServer'], `/api/user/prelogin`, model);
  }


  register(model) {
    let returnObs = this.http.post(environment.endpoints['IdentityServer'], `/api/user/register`, model).share();
    returnObs.subscribe((res: any) => {
      this.processLoginResponse(res);
    })
    return returnObs;
  }

  forgotPassword(model) {
    model.returnUrl = window.location.href;
    return this.http.post(environment.endpoints['IdentityServer'], `/api/user/forgot-password`, model);
  }

  processLoginResponse(res) {

    if (res.response == 'fill_data') {
      this.http.token = res.token;
    }

    if (res.response == 'logged_in') {
      console.log(res);
      this.http.token = res.token;
      this.currentUser = res.userAccount;
      this.cookieService.put('userAccount', JSON.stringify({ firstName: res.userAccount.firstName, lastName: res.userAccount.lastName, email: res.userAccount.email }));
      this.onUserLoggedIn.emit(res.userAccount);
      this.setLogOutTimer();

    }


  }
  processLoginToken(tkk) {
    return this.http.get(environment.endpoints['IdentityServer'], `/api/services/app/IdentityService/ProcessLoginToken?t=${tkk}`).subscribe(res => {
      if (res.result.type == 'TOKEN') {
        this.processOtpToken(res.result);
      }
    });
  }

  processOtpToken(result) {
    this.http.token = result.authToken;
    this.currentUser = result.user;
    this.onUserLoggedIn.emit({ action: result.action, user: this.currentUser });
    this.setLogOutTimer();
  }

  login(model) {
    let returnObs = this.http.post(environment.endpoints['IdentityServer'], `/api/user/login`, model).share();
    returnObs.subscribe((res: any) => {
      this.processLoginResponse(res);
    })

    return returnObs;
  }

  getOtp(model) {
    model.os = this.browser.getOS().name;
    model.osVersion = this.browser.getOS().versionName;
    model.userAgent = `${this.browser.getBrowser().name} ${this.browser.getBrowser().version}`;
    return this.http.post(environment.endpoints['IdentityServer'], `api/services/app/IdentityService/GetOtp`, model);
  }

  resendOtp(model) {
    model.os = this.browser.getOS().name;
    model.osVersion = this.browser.getOS().versionName;
    model.userAgent = `${this.browser.getBrowser().name} ${this.browser.getBrowser().version}`;

    return this.http.post(environment.endpoints['IdentityServer'], `api/services/app/IdentityService/ResendOtp`, model);

  }

  verifyOtp(model) {
    return this.http.post(environment.endpoints['IdentityServer'], `api/services/app/IdentityService/VerifyOtp`, model);
  }

  saveUserProfile(model) {
    return this.http.post(environment.endpoints['IdentityServer'], `api/services/app/IdentityService/SaveUserProfileAndLogin
`, model);
  }


  openLogin(clientFbPixel = null) {
    this.sidebarService.openSidebar(LoginFormComponent, { clientFbPixel: clientFbPixel });
  }

  getCurrentUserDetails(force = false) {
    if (!this.currentUser.id || force) {
      let obs2;
      let obs = this.http
        .get(environment.endpoints['IdentityServer'], '/api/user/me')
        .share();
      return obs;
    } else {
      return of(this.currentUser);
    }
  }


  getCurrentUserDetailsv2(force = false) {
    if (!this.currentUser.id || force) {
      let obs2;
      let obs = this.http
        .get(environment.endpoints['IdentityServer'], '/api/services/app/IdentityService/GetTokenModel')
        .share();
      return obs;
    } else {
      return of(this.currentUser);
    }
  }

  listUsers(filter: string) {
    let params = new HttpParams();
    if (filter && filter !== '')
      params.set('filterQuery', filter);

    return this.http.get(environment.endpoints['IdentityServer'], '/api/account/users', { search: params }).share();
  }


  logout(returnUrl: string = null) {
    this.cookieService.remove("op_token");
    let headers = new HttpHeaders();
    let loggedOutUser = this.currentUser;
    headers = headers.append('Content-Type', 'application/x-www-form-urlencoded');
    let postObj = { token: this.token, client_id: environment.client_id };
    this.cookieService.remove('auth_token');
    // this.http.post(environment.endpoints['IdentityServer'], '/connect/revocation', this.http.serializeToQueryString(postObj), { headers: headers } );
    this.currentUser = { claims: [] };
    this.currentUserObs = null;
    this.http.token = null;
    this.onUserLoggedOut.emit(loggedOutUser);

  }

  getDecodedAccessToken(token: string): any {
    let arrSplit = token.split('|');
    return { token: arrSplit[0], exp: arrSplit.length > 1 ? arrSplit[1] : null };
  }

  setLogOutTimer() {
    let tokenInfo = this.getDecodedAccessToken(this.token);
    if (!tokenInfo.exp)
      return;

    let expireDate: any = moment.unix(tokenInfo.exp);
    let currentDate: any = moment();
    let timeOut = expireDate.diff(currentDate);

    if (timeOut < 0)
      this.logout();

    if (this.logoutTimer) { this.logoutTimer.unsubscribe(); this.logoutTimer = null; }

    this.logoutTimer = timer(timeOut).subscribe(res => {
      this.logout()
    });
  }




  getUserClaims() {
    let obs = this.http.get(environment.endpoints['IdentityServer'], '/connect/userinfo').map(res => {
      try {
        return res.json();
      } catch (err) { }
    }).share();
    return obs;
  }

  getLoginUrl(clientId: string = environment.client_id, includePrefix: boolean = true, returnLocation: string = null, redirectUri = null) {
    if (!returnLocation) {
      returnLocation = window.location.pathname + window.location.search;
    }
    return (includePrefix ? environment.endpoints['IdentityServer'] : '') + `/connect/authorize?language=${this.translationService.currentLanguage}&client_id=${encodeURIComponent(clientId)}&redirect_uri=${encodeURIComponent((redirectUri ? redirectUri : window.location.protocol + '//' + window.location.host + '/login'))}&response_type=${encodeURIComponent('id_token token')}&scope=${encodeURIComponent(environment.identityserver_scopes)}&state=${encodeURIComponent(btoa(returnLocation))}&nonce=xyz`;
  }

  getCreateAccountUrl(clientId: string = environment.client_id, includePrefix: boolean = true) {
    return (includePrefix ? environment.endpoints['IdentityServer'] : '') + `/Account/Register?client_id=${clientId}&language=${this.translationService.currentLanguage}&returnUrl=${encodeURIComponent(this.getLoginUrl(clientId, false))}`;
  }

  getRememberPasswordUrl(clientId: string = environment.client_id, includePrefix: boolean = true) {
    return (includePrefix ? environment.endpoints['IdentityServer'] : '') + `/Account/ForgotPassword?client_id=${clientId}&language=${this.translationService.currentLanguage}&returnUrl=${encodeURIComponent(this.getLoginUrl(clientId, false))}`;
  }

  getLogoutUrl(returnUrl = null, siteUrl = null) {
    if (!returnUrl) {
      returnUrl = window.location.href;
    }

    return environment.endpoints['IdentityServer'] + `/account/logout?returnUrl=${encodeURIComponent(siteUrl ? siteUrl : (window.location.protocol + '//' + window.location.hostname + (window.location.port != 80 ? ':' + window.location.port : '') + '/logout?returnUrl=' + returnUrl))}&logoutId=1`;
  }

  processLogin(hash = null) {
    if (!hash)
      hash = window.location.hash.substr(1);
    let result = hash.split('&').reduce((result, item) => {
      let part0 = item.substring(0, item.indexOf('='));
      let part1 = item.substring(item.indexOf('=') + 1, item.length);

      result[part0] = part1;
      return result;
    }, {});
    if (result['error'] === 'access_denied') {
      this.router.navigateByUrl('/');
    }

    //this.http.token = result['access_token'];

    let obs = this.getCurrentUser(true);
    obs.subscribe(res => {
      this.currentUser.returnUrl = result['state'] ? atob(decodeURIComponent(result['state'])) : '/';
      //this.onUserLoggedIn.emit(this.currentUser);
    });
    return obs;
  }
  userHasPermission(perm) {
    return this.currentUser.claims.permission && this.currentUser.claims && this.currentUser.claims.permission.indexOf(perm) >= 0;
  }

  userHasPermissionv2(perm) {
    return this.currentUser.permissions && this.currentUser.permissions.includes(perm);
  }
  getExternalSignInUrl(provider, returnUrl = null) {
    if (provider == 'google')
      return `https://accounts.google.com/o/oauth2/v2/auth?client_id=${environment.google.clientId}&redirect_uri=${encodeURIComponent(window.location.protocol + '//' + window.location.host + '/login')}&scope=${encodeURIComponent('https://www.googleapis.com/auth/userinfo.email https://www.googleapis.com/auth/userinfo.profile')}&response_type=token&state=${encodeURIComponent(`google|${returnUrl}`)}`;
    else if (provider == 'facebook')
      return `https://www.facebook.com/v10.0/dialog/oauth?client_id=${environment.facebook.appId}&redirect_uri=${encodeURIComponent(window.location.protocol + '//' + window.location.host + '/login')}&state=${encodeURIComponent(`facebook|${returnUrl}`)}&display=popup
      &response_type=token`;
    else
      return environment.endpoints['IdentityServer'] + `/Account/ExternalLogin?provider=${encodeURIComponent(provider)}&returnUrl=${encodeURIComponent(this.getLoginUrl(environment.client_id, false, this.redirectTo, returnUrl))}`;
  }

  updatePassword(model) {
    return this.http.post(environment.endpoints['IdentityServer'], '/api/user/reset-password', model);

  }
  checkCurentUserClaimsv2(route) {
    let obs = this.getCurrentUserv2(!this.currentUser.id);
    return obs.pipe(map((result: any) => {
      let retValue = !route.data.permissions;//currentRouteConfig.canActivate.length > 0;
      if (!route.data.permissions || !this.currentUser.id)
        return retValue;
      if (route.data.permissions && route.data.permissions.length == 0 && this.currentUser.id)
        return true;
      console.log(result);
      route.data.permissions.forEach(x => {
        retValue = retValue || (result?.permissions?.indexOf(x) >= 0);
      });
      if (!retValue) {
        this.redirectTo = window.location.pathname + window.location.search;
        if (result)
          this.authError = 'Nu aveti suficiente drepturi pentru a accesa pagina dorita';
      } else {
        this.authError = undefined;
      }
      return retValue ? true : false;
    }));

  }

  checkCurentUserClaims(route) {
    let obs = this.getCurrentUser(!this.currentUser.id);
    return obs.pipe(map((result: any) => {
      let retValue = !route.data.permissions;//currentRouteConfig.canActivate.length > 0;
      if (!route.data.permissions || !this.currentUser.id)
        return retValue;
      if (route.data.permissions && route.data.permissions.length == 0 && this.currentUser.id)
        return true;
      route.data.permissions.forEach(x => {
        retValue = retValue || (result && result.claims && result.claims.permission && result.claims.permission.indexOf(x) >= 0);
      });
      if (!retValue) {
        this.redirectTo = window.location.pathname + window.location.search;
        if (result)
          this.authError = 'Nu aveti suficiente drepturi pentru a accesa pagina dorita';
      } else {
        this.authError = undefined;
      }
      return retValue ? true : false;
    }));

  }

  takeExternalLogin(obj, provider) {
    this.http.token = null;
    let providerString: string;
    switch (provider) {
      case `google`:
        providerString = environment.google.provider
        break;
      case `facebook`:
        providerString = environment.facebook.provider
        break;
      case `apple`:
        providerString = environment.apple.provider;
        if (window.cordova)
          obj.redirectUri = environment.apple.redirectUri;
        else
          obj.redirectUri = window.location.protocol + '//' + window.location.host + '/';

        break;
    }
    let obs = this.http.post(environment.endpoints['IdentityServer'], `/api/user/external-login?providerKey=${encodeURIComponent(providerString)}&clientId=${encodeURIComponent(environment.client_id)}`, obj).share();
    obs.subscribe(res => this.processLoginResponse(res));
    return obs;
  }

  requestLoginNameChange(model) {
    return this.http.post(environment.endpoints['IdentityServer'], '/api/services/app/IdentityService/RequestLoginNameChange', model);
  }

}
