import { NgModule, ModuleWithProviders } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { HttpClientModule } from '@angular/common/http';
import { LoadingButtonComponent, PagerComponent, TranslationComponent, LoginComponent, LoaderComponent, HtmlPopupComponent, NotificationToasterComponent, NotificationDropdownComponent, MultistepWizardComponent, HtmlPageComponent, SidebarComponent, FlowconceptIcheckComponent, LogoutComponent, NotificationListComponent, QuestionsComponent, LoginFormComponent, QuestionsAnswersComponent, CookieBarComponent, QrScannerComponent, LoginFormNewComponent, ChangeLoginComponent, InternationalPhoneNumberComponent } from './components';
import { MomentModule } from 'angular2-moment';
import { FlowHttpClient, GlobalService, ApplicationSharedService, UserAccountService, ClaimsGuardService, TranslationSharedService, PagerService, SignalrNotificationService, ModalService, AppSettingsLoaderService, SidebarService, TitleService } from './services';
import { RouterModule } from '@angular/router';
import { SumPipe, TranslationPipe } from './pipes';
import { ToDateTimePipe, HighlightPipe, CompareDirective, CnpValidationDirective, IcheckDirective, DatetimepickerDirective } from './directives';
import { ApplicationConfig } from './applicationConfig';
import { ModalModule } from 'ngx-modialog-7';
import { BootstrapModalModule, bootstrap3Mode } from 'ngx-modialog-7/plugins/bootstrap';
import { FileUploadModule } from 'ng2-file-upload';
import { FlowUploaderComponent } from './components/flow-uploader/flow-uploader.component';
import { Facebook } from '@awesome-cordova-plugins/facebook/ngx';
import { GooglePlus } from '@awesome-cordova-plugins/google-plus/ngx';
import { FlowCookieService } from 'flow-concept-shared/services/cookie/cookie.service';
import { InAppBrowser } from '@awesome-cordova-plugins/in-app-browser/ngx';
import { CommonModule } from '@angular/common';
import { CodeInputModule } from 'angular-code-input';
import { OrderModule } from 'ngx-order-pipe';
import { CountdownModule } from 'ngx-countdown';

declare var configuration: ApplicationConfig;




const MODULES_IMPORT = [
    CommonModule,
    FormsModule,
    HttpClientModule,
    //   CookieModule.forRoot(),
    MomentModule,
    RouterModule,
    ModalModule.forRoot(),
    BootstrapModalModule,
    OrderModule,
    FileUploadModule,
    CodeInputModule,
    CountdownModule,

    //   TooltipModule,
];
const MODULES_EXPORT = [
    FormsModule,
    HttpClientModule,
    //    CookieModule,
    MomentModule,
    ModalModule,
    BootstrapModalModule,
    FileUploadModule,
    //    TooltipModule,
    RouterModule,
    CodeInputModule,
    OrderModule,
    CountdownModule,
];

const COMPONENTS = [
    LoadingButtonComponent,
    TranslationComponent,
    PagerComponent,
    LoginComponent,
    LoaderComponent,
    HtmlPopupComponent,
    NotificationToasterComponent,
    NotificationDropdownComponent,
    MultistepWizardComponent,
    HtmlPageComponent,
    SidebarComponent,
    FlowconceptIcheckComponent,
    LogoutComponent,
    FlowUploaderComponent,
    NotificationListComponent,
    QuestionsComponent,
    LoginFormComponent,
    QuestionsAnswersComponent,
    CookieBarComponent,
    QrScannerComponent,
    LoginFormNewComponent,
    ChangeLoginComponent,
    InternationalPhoneNumberComponent
];

const SHARED_SERVICES = [
    ClaimsGuardService,
    GlobalService,
    ApplicationSharedService,
    UserAccountService,
    TranslationSharedService,
    PagerService,
    SignalrNotificationService,
    ModalService,
    AppSettingsLoaderService,
    SidebarService,
    FlowCookieService,
    TitleService
];

const SERVICES = [
    FlowHttpClient,
    Facebook,
    GooglePlus,
    InAppBrowser
];

const ENTRY_COMPONENTS = [
    LoginFormComponent,
];

const PIPES = [TranslationPipe, HighlightPipe, SumPipe];
const DIRECTIVES = [ToDateTimePipe, CompareDirective, CnpValidationDirective, IcheckDirective, DatetimepickerDirective];
bootstrap3Mode();
@NgModule({
    declarations: [
        ...COMPONENTS,
        ...PIPES,
        ...DIRECTIVES,
    ],
    imports: [
        ...MODULES_IMPORT,
    ],
    providers: [
        ...SERVICES
    ],
    exports: [
        ...MODULES_EXPORT,
        ...COMPONENTS,
        ...PIPES,
        ...DIRECTIVES
    ]
})
export class SharedModule {
    static forRoot(): ModuleWithProviders<SharedModule> {
        return {
            ngModule: SharedModule
        };
    }
}

@NgModule({
    providers: [
        ...SHARED_SERVICES
    ]
})
export class SharedServicesModule { }
